/*

* Copyright (c) 2005, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

package StatusSender;

import javax.microedition.midlet.*;
import javax.microedition.io.*;
import javax.microedition.lcdui.*;
import java.io.*;

public class Sender implements Runnable, CommandListener {
	
	private StatusSenderMIDlet parent;
	private Display display;
	private Command exitCommand = new Command("Exit", Command.EXIT, 1);
	private String ConnectionURL;
	private StringItem si;
	
	InputStream is;
    OutputStream os;
    SocketConnection sc;
	
	public Sender (StatusSenderMIDlet m, String ConnectionURL) 
	  {
	     parent = m;
	     this.ConnectionURL = ConnectionURL;	     
	     display = Display.getDisplay(parent);
	     
	     Form f = new Form("StatusSender");
         si = new StringItem("Status:","Connecting to server");
         f.append(si);
         
         f.addCommand(exitCommand);
         f.setCommandListener(this);
         display.setCurrent(f);	
         
	  }
	  
	
	public void start() 
	  {
        Thread t = new Thread(this);
        t.start();
      }
	
	public void run() 
	  {
		
		 try 
           {
             
             sc = (SocketConnection) Connector.open(ConnectionURL);
             is = sc.openInputStream();
             os = sc.openOutputStream();
             si.setText("Connected to server");
             
             int freeMem = (int)Runtime.getRuntime().freeMemory();            
             os.write(freeMem);  
             
             int c;
             c = is.read(); // server feeds back, indicating sucessfully receiving the physical status message sent

             if (c == -1) 
               {
               	
               	 Alert a = new Alert("StatusSender", "Fail to send the current physical status of the phone to the Aorta server. Please try again.",null, AlertType.ERROR);
                 a.setTimeout(10000);
                 a.addCommand(exitCommand);
                 a.setCommandListener(this);
                 display.setCurrent(a);
               	
               }
             else 
               {
               	
               	 Alert a = new Alert("StatusSender", "Successfully send the current physical status of the phone to the Aorta server",null, AlertType.INFO);
                 a.setTimeout(10000);
                 a.addCommand(exitCommand);
                 a.setCommandListener(this);
                 display.setCurrent(a);
               }               
             
           }  // try
         catch (ConnectionNotFoundException cnfe) 
           {
             Alert a = new Alert("StatusSender", "Fail to connect to the Aorta server. Please confirm the connection URL inputed is correct.",null, AlertType.ERROR);
             a.setTimeout(Alert.FOREVER);
             a.addCommand(exitCommand);
             a.setCommandListener(this);
             display.setCurrent(a);
           } 
         catch (IllegalArgumentException iae)
           {
          	 Alert a = new Alert("StatusSender", "Socket port number is missing in the inputed Aorta Server connection URL.", null, AlertType.ERROR);
             a.setTimeout(Alert.FOREVER);
             a.addCommand(exitCommand);
             a.setCommandListener(this);
             display.setCurrent(a);
           }
        catch (Exception e) {e.printStackTrace();}  		
      }  
    
    
    public void stop () 
      {
    	 try 
    	   {
    	     if (is != null) is.close();
    	
    	     if (os != null) os.close();
    	
    	     if (sc != null) sc.close();
           }
         catch (Exception e) {e.printStackTrace();}   	
      }  
	
	
	public void commandAction(Command c, Displayable s) 
	  {
		if (c == exitCommand)
		  {
		  	parent.notifyDestroyed();
            parent.destroyApp(true);
          }		
      } 	
}  