/*

* Copyright (c) 2005, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

package src.MessageDetector;


import javax.microedition.midlet.*;
import javax.microedition.io.*;
import javax.microedition.lcdui.*;
import java.io.*;

public class Client implements Runnable, CommandListener 
  {
    private SocketMIDlet parent;
    private Display display;
    private Form f;
    private StringItem si;
    private TextField tf;
    private boolean stop;
    private Command sendCommand = new Command("Send", Command.ITEM, 1);
    private Command exitCommand = new Command("Exit", Command.EXIT, 1);
    InputStream is;
    OutputStream os;
    SocketConnection sc;
    Sender sender;

    public Client(SocketMIDlet m) 
      {
        parent = m;
        display = Display.getDisplay(parent);
        f = new Form("Socket Client");
        si = new StringItem("Status:", " ");
        tf = new TextField("Send:", "", 30, TextField.ANY);
        f.append(si);
        f.append(tf);
        f.addCommand(exitCommand);
        f.addCommand(sendCommand);
        f.setCommandListener(this);
        display.setCurrent(f);
      }

    /* Start the client thread */
    public void start()
      {
        Thread t = new Thread(this);
        t.start();
      }

    public void run() 
      {
        try 
          {
            sc = (SocketConnection) Connector.open("socket://localhost:9000");
            si.setText("Connected to server");
            is = sc.openInputStream();
            os = sc.openOutputStream();

            // start the thread for sending messages - see Sender's main comment for explanation
            sender = new Sender(os);

            // loop forever, receiving data
            while (true) 
              {
                StringBuffer sb = new StringBuffer();
                int c = 0;

                while (((c = is.read()) != '\n') && (c != -1)) 
                    sb.append((char) c);                

                if (c == -1) break;

                // display message to user
                si.setText("Message received - " + sb.toString());
              }
            stop();
            si.setText("Connection closed");
            f.removeCommand(sendCommand);
          } 
        catch (ConnectionNotFoundException cnfe) 
          {
            Alert a = new Alert("Client", "Please run Server MIDlet first",null, AlertType.ERROR); 
            a.setTimeout(Alert.FOREVER);
            a.setCommandListener(this);
            display.setCurrent(a);
          } 
        catch (IOException ioe) 
          {
            if (!stop) ioe.printStackTrace();
            
          } 
        catch (Exception e) 
          {
            e.printStackTrace();
          }
      }

    public void commandAction(Command c, Displayable s) 
      {
        if (c == sendCommand && !parent.isPaused()) 
          sender.send(tf.getString());
        

        if ((c == Alert.DISMISS_COMMAND) || (c == exitCommand)) 
          {
            parent.notifyDestroyed();
            parent.destroyApp(true);
          }
      }

    /* Close all open streams */
    public void stop() 
      {
        try 
          {
            stop = true;

            if (sender != null) sender.stop();           

            if (is != null) is.close();
            

            if (os != null) os.close();
            
            if (sc != null) sc.close();
            
          } 
        catch (IOException ioe) {}
      }
}
