/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.util.*;
import java.io.*;
import net.tinyos.tinydb.*;
import net.tinyos.tinydb.parser.*;

/**
 * TinyDB sensor query acquires sensor tuple from a network of Crossbow motes.
 *
 * @see ScanOperator
 */
public class AortaSensorQuery implements ResultListener {

    /** Query class provided by the TinyDB API. */
    TinyDBQuery q;
    
    boolean aggregate;
    String aggValue;
    String aggOp;
    
    /** Scan operator this simulated sensor query outputs sensor tuple to. */
    public ScanOperator scanOp;
    /** SQL text of this simulated sensor query. */
    public String sensorQueryString;

    public AortaSensorQuery(String sensorQueryString, ScanOperator scanOp) 
      {
    	this.scanOp = scanOp;
	    this.sensorQueryString = sensorQueryString;	    
	    
	    try 
	      {
	        int index;       
	        String str;
	     
	        index = sensorQueryString.indexOf(";");
	        if (index == -1) // non-aggregate query
	          {
	       	    aggregate = false;
	       	    str = sensorQueryString;
	          }
	        else
	          {
	       	    aggregate = true;  // aggregate query
	            int start = sensorQueryString.indexOf(";");
	            start = start + 1;
	            int stop = sensorQueryString.indexOf(";",start);
	            aggValue = sensorQueryString.substring(start,stop);
	            str = sensorQueryString.substring(0,start-1);
	            aggOp = sensorQueryString.substring(stop + 1);
	          }	     
       	      
	     
	        PrintStream out_t1, out_t2, err_t;
	        out_t1 = System.out;
	        err_t = System.err;
	        File df = new File(".temp.out_err");
	        out_t2 = new PrintStream (new FileOutputStream(df));
	        System.setOut(out_t2);
	        System.setErr(out_t2);
	     
	        q = null;
	        q = SensorQueryer.translateQuery(str, scanOp.q.id);
	        if (q == null) System.out.println("fail!");	    	    
	    
	        TinyDBMain.injectQuery( q, this);
	     
	        System.setOut(out_t1); 
	        System.setErr(err_t);
	        out_t2.close();
	        df.delete();	     	          
	      } 
	    catch (Exception e) {e.printStackTrace();}
      }
     
    
    /**
     * When a sensor tuple is returned by TinyDB, output it to {@link #scanOp}.
     */
    public void addResult(QueryResult qr) 
      {
        scanOp.q.eventStartTime = System.currentTimeMillis();
	    scanOp.q.logFileWriter.println("An event occurs!");
		
	
	    // the timer function in the WHERE clause is unsatisfied now
	    if (!scanOp.q.whereTimerFuncName.equals(Aorta.NULL_STRING))
	     if (!scanOp.q.WHERE_TIMER_OPTIMIZATION)
           if (!scanOp.q.testTimerCondition())
             return;
	    
	    Vector v = qr.resultVector();
	    scanOp.q.logFileWriter.print("Sensor tuple: ");
	    
	    scanOp.q.logFileWriter.println();
	    scanOp.q.logFileWriter.println("Sensor query string: " + sensorQueryString);
	    scanOp.q.logFileWriter.flush();	
	
	    if (aggregate)
	      {
	        String s = (String)v.elementAt(1);
	      
	        if ( (aggOp.equals(">")) && (Integer.parseInt(s) <= Integer.parseInt(aggValue)) ) return;
	        else if ( (aggOp.equals(">=")) && (Integer.parseInt(s) < Integer.parseInt(aggValue)) ) return;
	             else if ( (aggOp.equals("<")) && (Integer.parseInt(s) >= Integer.parseInt(aggValue)) ) return;
	                  else if ( (aggOp.equals("<=")) && (Integer.parseInt(s) > Integer.parseInt(aggValue)) ) return; 
	                       else if ( (aggOp.equals("=")) && (Integer.parseInt(s) != Integer.parseInt(aggValue)) ) return;
	                            else if ( ((aggOp.equals("!=")) || (aggOp.equals("<>"))) && (Integer.parseInt(s) == Integer.parseInt(aggValue)) ) return;	
	    
	        Vector t = new Vector(10,1);
	        s = "0";
	        t.addElement((String)s);
	        t.addElement((String)s);
	        Tuple tuple = new Tuple (t);
	        scanOp.inputQueue.addElement((Tuple)tuple);
	        scanOp.getInput();	
	      }
	    else 
          {
       	    Tuple tuple = new Tuple (v);
       	    scanOp.inputQueue.addElement((Tuple)tuple);
	        scanOp.getInput();
          }
	
      } 

    
    /** Stop this TinyDB sensor query. */
    public void stopSensorQuery() 
      {
    	try {TinyDBMain.network.abortQuery(q);}
	    catch (Exception e) {e.printStackTrace();}
      } 
      
}
