/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * Thread for polling a real or simulated camera.
 *
 * @see CameraSimulator
 * @see ActionOperator#selectCandidateCamera
 * @see SchedulingThread
 */
public class CameraPollingThread extends Thread {
	
	CameraSimulator camSim;
	
	Tuple t;
	
	boolean cameraSimulation;
	
	boolean connected = false;
	
    public CameraPollingThread (CameraSimulator camSim, Tuple t, boolean cameraSimulation)
      {
      	this.camSim = camSim;
      	
      	this.t = t;
      	
      	this.cameraSimulation = cameraSimulation;
      }
      
      
    /** 
     * Poll a candidate camera and know the current physical status of the camera. 
     * Wait at most {@link CameraSimulator#POLLING_TIMEOUT_LIMIT} for the camera to response.
     **/
    public void run ()
      {
      	if (cameraSimulation == true)
      	  {
      	  	if (camSim.polling() != CameraSimulator.POLLING_TIMED_OUT)    	  	      	  	
      	      connected = true;
          }
      	else
      	  {
      	  	String userPassword = (String)(t.fields.elementAt(1)) + ":" + (String)(t.fields.elementAt(2));
      	    String encoding = new sun.misc.BASE64Encoder().encode (userPassword.getBytes());
            String camera_ip = (String)(t.fields.elementAt(3));
            
            camSim.pollingTime = System.currentTimeMillis();
            
            try
              {
          	    // System.setProperty("sun.net.client.defaultConnectTimeout",String.valueOf(CameraSimulator.POLLING_TIMEOUT_LIMIT)); 
                // System.setProperty("sun.net.client.defaultReadTimeout",String.valueOf(CameraSimulator.POLLING_TIMEOUT_LIMIT)); 
          	    URL url = new URL("http://" + camera_ip + "/axis-cgi/com/ptz.cgi?query=position"); 
      	        URLConnection uc = url.openConnection();
                uc.setRequestProperty  ("Authorization", "Basic " + encoding);               
                BufferedReader br = new BufferedReader (new InputStreamReader(uc.getInputStream()));
          	    
          	    camSim.pollingTime = System.currentTimeMillis() - camSim.pollingTime; 
          	    
          	    String str;
          	    int start;
          	    while ( (str = br.readLine()) != null)
          	      {
          	  	    /* Message format: camera = N \r\n pan = xx \r\n tilt = xx \r\n zoom = xx \r\n */
          	  	    if ( (start = str.indexOf("pan")) != -1)
          	  	      {
          	  	        start = str.indexOf("=",start)+1;
          	  	  	    camSim.pan = Integer.parseInt(str.substring(start).trim());
          	          }
          	      
          	        if ( (start = str.indexOf("tilt")) != -1)
          	          {
          	  	  	    start = str.indexOf("=",start)+1;
          	  	  	    camSim.tilt = Integer.parseInt(str.substring(start).trim());
          	          }
          	  	  	
          	  	    if ( (start = str.indexOf("zoom")) != -1)
          	          {
          	  	  	    start = str.indexOf("=",start)+1;
          	  	        camSim.zoom = Integer.parseInt(str.substring(start).trim());
          	          }             	      
          	      }
          	      
          	    connected = true;  // the candidate camera is currently available        	              	             	          	         	         	                 
              }
            catch (Exception e)  // socket timed out
		      { 
			    // System.setProperty("sun.net.client.defaultConnectTimeout","-1"); 
                // System.setProperty("sun.net.client.defaultReadTimeout","-1");
                camSim.pollingTime = CameraSimulator.POLLING_TIMED_OUT;               
		      }                  
        
            // System.setProperty("sun.net.client.defaultConnectTimeout","-1"); 
            // System.setProperty("sun.net.client.defaultReadTimeout","-1");                          
      	  }     	
      }
    	
}