/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/ 

import java.io.*;
import java.util.*;
import net.tinyos.tinydb.*;

/** 
 * A simple command line user interface for Aorta.
 *
 * @see Aorta
 */
public class CmdLineUserIF extends Thread {
	
    Aorta aortaqp;
        
    
    /** 
     * @param aortaqp the Aorta query processor that this command line user interface corresponds to
     */
    public CmdLineUserIF (Aorta aortaqp) 
      {
      	this.aortaqp = aortaqp;
      }
      
 
    
    /** 
     * Continuously read and parse a command from <code>System.in</code>.
     */
    public void run() 
      {
    	try {
      	
        int i,j;
        int start,stop;
        String cmd, cmd_origin;
        BufferedReader br = new BufferedReader (new InputStreamReader(System.in));
               
      
    	while (true)
    	  {
    	    // read a command line from System.in
    	    cmd = br.readLine(); 
    	    
    	    if (cmd == null)
    	      System.exit(0);
    	    
    	    cmd_origin = cmd;      	    
    	    cmd = cmd.toLowerCase();
    	    cmd = cmd.trim();
    	    cmd.replace('\t',' ');
    	    cmd.replace('\n',' ');
    	    cmd.replace('\r',' ');
    	    cmd.replaceAll("  ", " ");
    	    
    	    
            // empty command
    	    if (cmd.equals(""))
    	      {
    	      	System.out.print("Aorta>");
    	      	continue;
    	      }
    	      
    	      	    
    	    // CREATE command
      	  	start = cmd.indexOf("create");
            if (start == 0)
              {
              	 if (cmd.equals("create"))
                   {
                   	  System.err.println("Invalid command syntax. Argument AQ | ACTION is missing.");
                   	  System.out.print("Aorta>");
                   	  continue;
                   }
                 
                 if (cmd.charAt(6) != ' ')
                   {
                   	   System.err.println(cmd + ": command not found");
    	               System.out.print("Aorta>");
    	               continue;
    	           }
    	           
                 cmd = cmd + "          ";
                 start = start + 7;
                 if (cmd.substring(start,start+7).equals("action "))
                   {
                   	  cmd = cmd_origin;
                   	  start += 7;
                   	  stop = cmd.indexOf("(",start);
                   	  if (stop == -1)
                   	    {
                   	      System.err.println("Invalid syntax of the CREATE ACTION command");
                   	      System.out.print("Aorta>");
                   	      continue;
                   	    }
                   	  
                   	  UDAction action = new UDAction ();
                   	  String str = cmd.substring(start,stop).trim();
                   	  if (str.length() == 0)
                   	    {
                   	      System.err.println("Invalid syntax of the CREATE ACTION command"); 
                   	      System.out.print("Aorta>");
                   	      continue;
                   	    }
                   	  boolean dupName = false;                   	        	  
                   	  for (i=0;i<aortaqp.actionNum;i++)
                   	    if (str.equals(aortaqp.actions[i].name))
                   	      {
                   	        System.err.println("An action with the same name has been registered to Aorta."); 
                   	        System.out.print("Aorta>");
                   	        dupName = true;
                   	        break;
                   	      }
                   	  if (dupName)
                   	    continue;
                   	  action.name = str;                   	 
                   	                      	  
                   	  start = stop + 1;
                   	  stop = cmd.indexOf(")",start);
                   	  if (stop == -1)
                   	    {
                   	      System.err.println("Invalid syntax of the CREATE ACTION command"); 
                   	      System.out.print("Aorta>");                   	     
                   	      continue;
                   	    }
                   	  str = cmd.substring(start,stop).trim();
                   	  String paramString = str;
                   	  
                      boolean noParams = false;
                      if (cmd.substring(start,stop).trim().length() == 0) 
                        noParams = true;
                   	  
                   	  start = 0;
                   	  while (!noParams)
                   	    {
                   	      stop = paramString.indexOf(" ",start);
                   	      if (stop == -1)
                   	        {
                   	          System.err.println("Invalid syntax of the CREATE ACTION command"); 
                   	          break;
                   	        }
                   	      action.params[action.paramNum].type = paramString.substring(start,stop).trim();
                   	      start = stop + 1;
                   	      stop = paramString.indexOf(",",start);
                   	      if (stop != -1)
                   	        {
                   	          action.params[action.paramNum].name = paramString.substring(start,stop).trim(); 
                   	          action.paramNum++; 
                   	          start = stop + 1;
                   	          continue;
                   	        }
                   	      else
                   	        {
                   	          action.params[action.paramNum].name = paramString.substring(start);
                   	          action.paramNum++;
                   	          break;	
                   	        }                   	      
                   	    }                   	                    	  
                   	  
                   	  start = cmd.indexOf(")") + 1;
                   	  start = cmd.indexOf(" as ",start);
                   	  if (start == -1)
                   	    start = cmd.indexOf(" AS ", start);
                   	  if (start == -1)
                   	    {
                   	      System.err.println("Invalid syntax of the CREATE ACTION command"); 
                   	      System.out.print("Aorta>");
                   	      continue;
                   	    }
                   	  start += 4;
                   	  stop = cmd.indexOf(" profile ",start);
                   	  if (stop == -1)
                   	    stop = cmd.indexOf(" PROFILE ",start);
                   	  if (stop == -1)
                   	    {
                   	      System.err.println("Invalid syntax of the CREATE ACTION command");
                   	      System.out.print("Aorta>");
                   	      continue;
                   	    }
                   	  str = cmd.substring(start,stop).trim();
                   	  str = str.substring(1,str.length()-1);  // the pathname of the action class file
                   	  
                   	  File f = new File(str);
                   	  if (!f.exists())
                   	    {
                   	      System.err.println("Cannot locate the action class file:" + str + ".");
                   	      System.out.print("Aorta>");
                   	      continue;	
                   	    }                   	                     	   
                   	  
                   	  start = str.indexOf("/");
                   	  if (start != -1)
                   	    {
                   	      while (start != -1)
                   	        {
                   	          stop = start;
                   	          start = str.indexOf("/", start+1);	
                   	        }                   	  
                   	      start = stop;
                   	    }
                   	  else 
                   	    start = 0;
                   	  Runtime.getRuntime().exec("cp " + str + " " + str.substring(start));                   	  
                   	  str = str.substring(start);
                   	  stop = str.indexOf(".class");
                   	  if (stop == -1)
                   	    {
                   	      System.err.println("Invalid syntax of the CREATE ACTION command");
                   	      System.out.print("Aorta>");
                   	      continue;
                   	    }
                   	  str = str.substring(0,stop);
                   	  
                   	  Class c = Class.forName(str);                   	  
                   	  Object object = c.newInstance();
                   	  UDAction udAction = (UDAction)object;
                   	  
                   	  udAction.name = action.name;
                   	  udAction.params = action.params;
                   	  udAction.paramNum = action.paramNum;
                   	  
                   	  start = cmd.indexOf(" profile ");
                   	  start += 9;
                   	  str = cmd.substring(start).trim();
                   	  str = str.substring(1,str.length()-1);
                   	  f = new File (str);
                   	  if (!f.exists())
                   	    {
                   	      System.err.println("Cannot locate the action profile:" + str + ".");
                   	      System.out.print("Aorta>");
                   	      continue;	
                   	    }
                   	  udAction.profile = str;
                   	  udAction.flag = Action.USER_DEFINED;
                   	  
                   	  if (!udAction.parseProfile())
                   	    {
                   	      System.out.print("Aorta>");
                   	      continue;
                   	    }
                   	  
                   	  aortaqp.actions[aortaqp.actionNum++] = udAction;                  	  
                   	  
                   	  System.out.print("Successfully register the action " + udAction.name + "(");
                   	  for (i=0;i<udAction.paramNum;i++)
                   	    {
		      	  	      System.out.print(udAction.params[i].name);
		      	  	      if (i==udAction.paramNum - 1)
		      	  	        System.out.print(")");
		      	  	      else
		      	  	        System.out.print(",");
		      	  	    }
                   	  if (udAction.paramNum == 0)
                   	    System.out.println(")");
                   	  System.out.println(" to Aorta.");                   	 
                   	  System.out.print("Aorta>");
                   	  continue;
                   }
                                    
              	 aortaqp.addAQ(cmd.trim() + " ");
              	 System.out.print("Aorta>");
    	         continue; 
    	         
		      }
		      
		      
		    // DROP command
		    start = cmd.indexOf("drop");
		    if (start == 0)
		      {
		      	
		      	if (cmd.length() == 4)
                  {
                     System.err.println("Invalid command syntax. Argument AQ | ACTION is missing.");
                     System.out.print("Aorta>");
                     continue;
                  }
                  
                if (cmd.charAt(4) != ' ')
                  {
                   	   System.err.println(cmd + ": command not found");
    	               System.out.print("Aorta>");
    	               continue;
    	          }
		      	
		      	start = start + 5;
		      	cmd = cmd + "        ";
		      	if (cmd.substring(start,start+ 7).equals("action "))
		      	  {
		      	  	start += 7;
		      	  	String actionName = cmd.substring(start);
		      	  	for (i=0;i<aortaqp.actionNum;i++)
		      	  	  if (aortaqp.actions[i].name.equals(actionName))
		      	  	  	{
		      	  	  	  if (aortaqp.actions[i].flag == Action.SYSTEM_BUILT_IN)
		      	  	  	    System.out.println("Cannot drop system built-in action " + actionName + "().");
		      	  	  	  else
		      	  	  	    {
		      	  	  	      aortaqp.actions[i] = aortaqp.actions[aortaqp.actionNum-1];
		      	  	  	      aortaqp.actionNum--;
		      	  	  	      System.out.println("Successfully drop user-defined action " + actionName + "().");
		      	  	  	    }
		      	  	  	    
		      	  	  	  break;
		      	  	    }  	      	  	  
		      	  	
		      	  	System.out.print("Aorta>");
		      	  	continue;
		      	  }
		        else
		      	  if (!cmd.substring(start,start + 3).equals("aq "))
		      	     {
		      	    	System.err.println("Invalid command syntax.");
		      	    	System.out.print("Aorta>");
		      	    	continue;
		      	     }
		        
		        start = start + 3; 		      	
		      	String name = cmd.substring(start).trim();
		      	int size = aortaqp.AQVector.size();
		      	for (i=0;i<size;i++)
		      	  if (name.equals(((AQ)(aortaqp.AQVector.elementAt(i))).name))
		      	    { 
		      	      ((AQ)(aortaqp.AQVector.elementAt(i))).destroy();
		      	      aortaqp.AQVector.removeElementAt(i);
		      	      break;
		      	    }
		      	if (i == size)
		      	  System.err.println("There is no query named \"" + name + "\" that has been installed to Aorta.");
		      	else 
		      	  System.err.println("Successfully remove the query \"" + name + "\".");
		      	System.out.print("Aorta>");
    	        continue;
		      }
		    
		      
		    // ALTER command
		    start = cmd.indexOf("alter");
		    if (start == 0)
		      {
		      	if (cmd.length() == 5)
                  {
                     System.err.println("Invalid command syntax. Argument AQ | ACTION is missing.");
                     System.out.print("Aorta>");
                     continue;
                  }
                  
                if (cmd.charAt(5) != ' ')
                   {
                   	   System.err.println(cmd + ": command not found");
    	               System.out.print("Aorta>");
    	               continue;
    	           }
                  
                start = start + 6;
                cmd = cmd + "        ";
		      	if (cmd.substring(start,start+ 7).equals("action "))
		      	  {
		      	  	System.out.println("Invalid command syntax");
		      	  	System.out.print("Aorta>");
		      	  	continue;
		      	  }
		        else
		      	  if (!cmd.substring(start,start + 3).equals("aq "))
		      	     {
		      	    	System.err.println("Invalid command syntax.");
		      	    	System.out.print("Aorta>");
		      	    	continue;
		      	     }
		      	
		      	cmd = cmd.trim();     
                System.out.println("The ALTER AQ command is not yet implemetented.");    
		      	System.out.print("Aorta>");
    	        continue;
		      }
		      
		    
		    // LIST command, list the sql texts of all queries (LIST AQ) or list the specifications of all actions (LIST ACTION)
		    if (cmd.equals("list"))
		      {
		      	System.out.println("Invalid command syntax. Argument AQ | ACTION is missing.");
		      	System.out.print("Aorta>");
		      	continue;
		      }
		    if (cmd.equals("list aq"))
		      {
		      	
		      	if (aortaqp.AQVector.size() == 0)
		      	  {
		      	  	System.out.println("There is no query installed to Aorta now.");
		      	  	System.out.print("Aorta>");
		      	  	continue;
		      	  }
		      	System.out.println();
		      	System.out.println("The following action-embedded queries have been installed to Aorta:");
		      	System.out.println();
		      	AQ q;		      	
		      	for (i=0;i<aortaqp.AQVector.size();i++)
		      	  {
		      	  	q = (AQ)aortaqp.AQVector.elementAt(i);
		      	  	System.out.println("Query id " + q.id + ":");
		      	  	System.out.println("Query string: \r\n" + q.AQString);
		      	  	System.out.println();		      	  	
		      	  }
		      	System.out.print("Aorta>");
		      	continue;
		      }
		      
		    if (cmd.equals("list action"))
		      {
		      	System.out.println();
		      	System.out.println("The following actions have been registered to Aorta:");
		      	int count = 1;
		      	for (i=0;i<aortaqp.actionNum;i++)
		      	  {
		      	  	System.out.print(count + ". ");
		      	  	count ++;
		      	  	System.out.print(aortaqp.actions[i].name);
		      	  	System.out.print("(");
		      	  	for (j=0;j<aortaqp.actions[i].paramNum;j++)
		      	  	  {
		      	  	    System.out.print(aortaqp.actions[i].params[j].name);
		      	  	    if (j==aortaqp.actions[i].paramNum - 1)
		      	  	      System.out.print(")");
		      	  	    else
		      	  	      System.out.print(",");
		      	  	  }
		      	  	if (aortaqp.actions[i].paramNum == 0)
		      	  	  System.out.print(")");
		      	  	System.out.println();
		      	  }		      	
		      	System.out.println();
		      	System.out.print("Aorta>");
		      	continue;
		      }
		    
		    
		    // HELP command, which lists all valid commands in AortaSQL and give incividual examples to show how to use them
		    if (cmd.equals("help")) 
		      {
		      	System.out.println();
		      	System.out.println("AortaSQL includes the following five commands:");
		      	System.out.println();
		      	System.out.println("CREATE\t\tDROP\t\tALTER\t\tLIST\t\tEXIT");
		      	System.out.println();		      	
		      	System.out.println("CREATE AQ: create a new action-embedded query (AQ) and install it to Aorta.");
		      	System.out.println("Example: CREATE AQ night_surveillance AS SELECT sendphoto(p.number, photo(c.ip, \"images\")) FROM sensor s, camera c, phone p WHERE  s.accel_x > 600 AND dist(s.loc, c.loc) < 5 AND p.owner = \"admin\"  START atTime(0,0,0) STOP atTime(7,0,0) LIFETIME 2 months");
		      	System.out.println();
		      	System.out.println("CREATE ACTION: register a new user-defined action to Aorta.");
		      	System.out.println("Example: CREATE ACTION sendphoto(String phone_no, String photo_pathname) AS \"admin/lib/sendphoto.class\" PROFILE \"admin/profiles/sendphoto.xml\"");
		      	System.out.println();
		      	System.out.println("DROP AQ: drop (remove) a pre-installed AQ.");
		      	System.out.println("Example: DROP AQ night_surveillance");
		      	System.out.println();
		      	System.out.println("DROP ACTION: drop a user-defined action.");
		      	System.out.println("Example: DROP ACTION sendphoto");
		      	System.out.println();
		      	System.out.println("ALTER AQ: alter the specification of a pre-installed AQ.");
		      	System.out.println("Example: ALTER AQ night_surveillance SET START $now");
		      	System.out.println();
		      	System.out.println("LIST AQ: list all AQs that have been installed to Aorta.");
		      	System.out.println();
		      	System.out.println("LIST ACTION: list all actions that have been registered to Aorta.");
		      	System.out.println();
		      	System.out.println("HELP: list all commands in AortaSQL with illustrative examples.");
		      	System.out.println();
		      	System.out.println("EXIT: exit the system.");
		      	System.out.println();
		      	System.out.print("Aorta>");
    	        continue;
		      }
		    
		    
		    // EXIT command, terminate the system   
    	    if (cmd.equals("exit"))
    	      {
    	        br.close();
                System.out.println("Thanks for using Aorta. Goodbye!");
    	        for (i=0; i<aortaqp.AQVector.size();i++)
                  ((AQ)aortaqp.AQVector.elementAt(i)).destroy();
                System.exit(0);	
    	      }
    	    
    	    // invalid command
    	    System.err.println(cmd + ": command not found");
    	    System.out.print("Aorta>");
    	    	        	    
    	  }  
    	} 
        catch (Exception e) {e.printStackTrace();}
      } 	
  	
}