/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;

/**
 * Join operator in AQ plan.
 *
 * @see AQ
 */
public class JoinOperator extends Operator {
	
	public CacheEntry cache [] = new CacheEntry [CacheEntry.CACHE_SIZE];
	
	int cache_num = 0;
	
	/* Private variables */
	private Tuple t;
	private String did;
	
	public JoinOperator (AQ q)
	  {
	  	type = JOIN;
	  	this.q = q;
	  }
	  
	public void start() {}
	  
	public void stop() {}
	  
	public void getInput()
	  {
	  	int i;
	  	Vector v = null;
	  	
	  	t = deQueue(1);		
		if (q.scanOp != null && q.scanOp.outputOperator == this)
		  did = (String)t.fields.elementAt(1);
		else
		  did = (String)t.fields.elementAt(0);		
	  
		if (q.JOIN_OPTIMIZATION)
		  { 
		  	for (i=0;i<cache_num;i++)
		  	  if (cache[i].value.equals(did))
		  	    {v=cache[i].tuples; cache[i].lruValue = System.currentTimeMillis(); break;}
		  }
		
		if (!q.JOIN_OPTIMIZATION || v == null)
		  { 
		  	v = findDeviceIDJoin(); 
		  	if (q.JOIN_OPTIMIZATION)
		      if (cache_num < CacheEntry.CACHE_SIZE - 1)  // if the cache is not full
		        {
		        	cache[cache_num] = new CacheEntry(did,v);
		        	cache[cache_num++].lruValue = System.currentTimeMillis();
		        }
		      else
		        {
		          long lru = Long.MAX_VALUE;
		          int no = -1;
		          for (i=0;i<cache_num;i++)
		            if (cache[i].lruValue < lru)
		              no = i;
		          cache[i] = new CacheEntry(did,v);
		          cache[i].lruValue = System.currentTimeMillis(); 
		        } 	  
          }
				  
		for (i=0;i<v.size();i++)  
		  {
		    t = (Tuple)v.elementAt(i);
		    enQueue(t);	
          }
        
        outputOperator.getInput();        
	  }
	  
	
	/** 
	 * Given two join attributes and the join predicate, find the corresponding pairs of 
	 * two devices that satisfy the join predicate.
	 *
	 * @return a vector each element of which is a tuple that encapsulates the information 
	 *         required to execute the action about a join of two devices
	 *
	 * @see Tuple
	 */
	public Vector findDeviceIDJoin() 
	  {
	     String deviceBaseStation1 = null, deviceBaseStation2 = null;
	     String deviceJoinAttrV1 = null;
         String str;
         int start, stop;
         Vector v = new Vector (5,5);
         BufferedReader br;
      
         if (q.triggeringJoinAttrN.indexOf(Aorta.SENSOR_SHORT) != -1)
           deviceBaseStation1 = Aorta.SENSOR_BASESTATION;
         else
           if (q.triggeringJoinAttrN.indexOf(Aorta.CAMERA_SHORT) != -1)
             deviceBaseStation1 = Aorta.CAMERA_BASESTATION;
           else
             if (q.triggeringJoinAttrN.indexOf(Aorta.PHONE_SHORT) != -1)
               deviceBaseStation1 = Aorta.PHONE_BASESTATION;
             else
               if (q.triggeringJoinAttrN.indexOf(Aorta.ROBOT_SHORT) != -1)
                 deviceBaseStation1 = Aorta.ROBOT_BASESTATION;
            
         if (q.triggeredJoinAttrN.indexOf(Aorta.SENSOR_SHORT) != -1)
           deviceBaseStation2 = Aorta.SENSOR_BASESTATION;
         else
           if (q.triggeredJoinAttrN.indexOf(Aorta.CAMERA_SHORT) != -1)
             deviceBaseStation2 = Aorta.CAMERA_BASESTATION;
           else
             if (q.triggeredJoinAttrN.indexOf(Aorta.PHONE_SHORT) != -1)
               deviceBaseStation2 = Aorta.PHONE_BASESTATION;
             else
               if (q.triggeredJoinAttrN.indexOf(Aorta.ROBOT_SHORT) != -1)
                 deviceBaseStation2 = Aorta.ROBOT_BASESTATION;
                	
	     try 
	       { 
		      if (q.selOp != null && q.selOp.outputOperator == this)
		        deviceJoinAttrV1 = (String)t.fields.elementAt(1);
		      else 
		        {		    
		          br = new BufferedReader(new FileReader(deviceBaseStation1));          
                  while ((str = br.readLine()) != null) 
                    {
                      str = str.toLowerCase().trim();
               
                      start = str.indexOf("<id>");
                      if (start == -1) continue; 
                      start += "<id>".length();              
                      stop = str.indexOf("</id>",start);               
          
                      if (!did.equals(str.substring(start,stop).trim())) continue;
                      else 
                        {
            	          start = str.indexOf("<"+q.triggeringJoinAttrN.substring(2)+">", stop);
            	          if (start == -1) 
            	            {
            	  	          System.err.println("Unknown sensor Attribute "+ q.triggeringJoinAttrN.substring(2)+ ".");
	      	                  System.exit(1);
            	            }
            	          start = str.indexOf(">",start);
            	          start = start + 1;
            	          stop = str.indexOf("</",start);
            	          deviceJoinAttrV1 = str.substring(start,stop).trim(); 
            	          break;        	 
                        }                           
                    } 	 
	              if (deviceJoinAttrV1 == null) 
	                return new Vector (1,0);	         
	              br.close();
	            }
	      
	      
	          br = new BufferedReader(new FileReader(deviceBaseStation2));        
	          while ((str = br.readLine()) != null) 
	            {
                  str = str.toLowerCase().trim();
              
                  start = str.indexOf("<model>");
                  if (start == -1) continue;
                  start += "<model>".length();
                  stop = str.indexOf("</model>",start);
                  if (stop == -1) continue;
              
                  if (this.outputOperator == q.actionOp)
                    if (!str.substring(start,stop).trim().equals(q.action.devInf.model.toLowerCase()))
                      continue;                  
                  if (this.outputOperator == q.actionOp2)
                    if (!str.substring(start,stop).trim().equals(q.action2.devInf.model.toLowerCase()))
                      continue;
              
                  start = str.indexOf("<"+q.triggeredJoinAttrN.substring(2) + ">");
                  if (start == -1) continue;
          
                  start = str.indexOf(">",start);
                  start = start + 1;
                  stop = str.indexOf("</", start);
                  String s = str.substring(start, stop).trim();
             
                  // currently we only support one join function: dist(loc1, loc2) < distant (unit: meter)
                  if (!q.joinFuncName.equals("dist")) 
                    continue;
                  else
                    {
                      if (q.joinArithOp.equals("<"))
                        if ((Aorta.dist(Point.parsePoint(deviceJoinAttrV1),Point.parsePoint(s)) < Double.parseDouble(q.joinConstant)) == false)
                          continue;
                      if (q.joinArithOp.equals(">"))
                        if ((Aorta.dist(Point.parsePoint(deviceJoinAttrV1),Point.parsePoint(s)) > Double.parseDouble(q.joinConstant)) == false)
                          continue;
                      if (q.joinArithOp.equals("<="))
                        if ((Aorta.dist(Point.parsePoint(deviceJoinAttrV1),Point.parsePoint(s)) <= Double.parseDouble(q.joinConstant)) == false)
                          continue;
                      if (q.joinArithOp.equals(">="))
                        if ((Aorta.dist(Point.parsePoint(deviceJoinAttrV1),Point.parsePoint(s)) >= Double.parseDouble(q.joinConstant)) == false)
                          continue;
                      if (q.joinArithOp.equals("="))
                        if ((Aorta.dist(Point.parsePoint(deviceJoinAttrV1),Point.parsePoint(s)) == Double.parseDouble(q.joinConstant)) == false)
                          continue;
                      if (q.joinArithOp.equals("!=") || q.joinArithOp.equals("<>"))
                        if ((Aorta.dist(Point.parsePoint(deviceJoinAttrV1),Point.parsePoint(s)) != Double.parseDouble(q.joinConstant)) == false)
                          continue;
                      
                      start = str.lastIndexOf("<id>", start);
            	      if (start == -1) 
            	        {
            	  	      System.err.println("Invalid format of the camera base station.");
	      	              System.exit(1);
            	        }
            	      start = str.indexOf(">",start);
            	      start = start + 1;
            	      stop = str.indexOf("</",start);
            	      did = str.substring(start,stop); 
            	      Vector vt = new Vector(5,1);
            	      vt.addElement(did);  // id of the candidate device
            	  
            	      // authorization information about the camera: username + password
            	      start = str.indexOf("<username>");
            	      if (start == -1) 
            	        {
            	  	      System.err.println("Invalid format of the camera base station.");
	      	              System.exit(1);
            	        }
            	      stop = str.indexOf("</",start);
            	      vt.addElement(str.substring(start+10,stop).trim());
            	      start = str.indexOf("<password>",start);
            	      if (start == -1) 
            	        {
            	  	      System.err.println("Invalid format of the camera base station.");
	      	              System.exit(1);
            	        }
            	      stop = str.indexOf("</",start);
            	      vt.addElement(str.substring(start+10,stop).trim());
            	  
            	      // decide the instantiated values of action input parameters
            	      String actionParams [];
            	      if (this.outputOperator == q.actionOp2)
            	        actionParams = q.actionParams2;
            	      else
                        actionParams = q.actionParams;
            	      int i =0;
            	      while (actionParams[i] != null)
            	        {
            	          if (actionParams[i].indexOf(q.triggeredJoinAttrN.substring(0,2)) != -1)
            	            {
            	              start = str.indexOf("<" + actionParams[i].substring(2) + ">");
            	              if (start != -1)
            	                {
            	                  start = str.indexOf(">",start) + 1;
            	                  stop = str.indexOf("</",start);
            	                  vt.addElement(str.substring(start,stop).trim());	
            	                }	
            	            }            	      
            	          i++;	
            	        }              	
            	    
            	      vt.addElement(deviceJoinAttrV1);
            	  
            	      Tuple t = new Tuple(vt);
            	      t.qid = q.id;
            	      t.seqNo = q.aortaqp.seqNo;
            	             	   	         
            	      // e.g., the fields of the tuple are: camera_id camera_username camera_password camera_ip sensor_loc  
            	      v.addElement(t);           	               	 	          	            
                    }  
          
                }  
          
	          q.aortaqp.seqNo++;
	      
	          br.close();       
	       } 
	     catch (Exception e) {e.printStackTrace();}
	   
	     return v;	
      } 		
	  
} 