/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/


/**
 * Thread class that continuously monitors the LIFETIME clause of an AQ.
 *
 * @see AQ
 * @see TimerFunctions
 */
public class LifeTimeClause extends Thread {
	
	/** The AQ that this LIFETIME clause belongs to. */
	public AQ q;
	
    /** @param q the AQ that this LIEFTIME clause belongs to. */
    public LifeTimeClause (AQ q)
	  {
	  	this.q = q;
	  }
	  
	
	/**
	 * Based on the time unit and value specified, set a one-time timer that triggers
	 * the removal of {@link #q} in the future.
	 */
	public void run()
	  {
	  	long sleep_interval = 0;
	  	
	  	if (q.lifeTimeUnit.equals(TimerFunctions.YEAR))
	  	  sleep_interval = q.lifeTime * 365 * TimerFunctions.ONE_DAY_SECONDS;
	  	else if (q.lifeTimeUnit.equals(TimerFunctions.MONTH))
	  	       sleep_interval = (long)(q.lifeTime * 30.5 * TimerFunctions.ONE_DAY_SECONDS);
	  	     else if (q.lifeTimeUnit.equals(TimerFunctions.DAY))
	  	            sleep_interval = q.lifeTime * TimerFunctions.ONE_DAY_SECONDS;
	  	          else if (q.lifeTimeUnit.equals(TimerFunctions.HOUR))
	  	                 sleep_interval = q.lifeTime * 60 * 60;
	  	               else if (q.lifeTimeUnit.equals(TimerFunctions.MINUTE))
	  	                      sleep_interval = q.lifeTime * 60;
	  	                    else if (q.lifeTimeUnit.equals(TimerFunctions.SECOND))
	  	                           sleep_interval = q.lifeTime;
	  	
	  	sleep_interval *= 1000;
	  	
	  	try 
	  	  {
	  	  	while (true)
	  	  	  {
	  	  	  	Thread.currentThread().sleep(sleep_interval);
	  	  	    sleep_interval -= q.startTime;
	  	  	    if (sleep_interval > 0)
	  	  	      Thread.currentThread().sleep(sleep_interval);
	  	  	    else 
	  	  	      break;
	  	  	  }
	  	  }
	  	catch (Exception e) {e.printStackTrace();}
	  	q.stop();
	    System.out.println("Query " + q.name + " (ID:" + q.id + ") expired just now. It has been removed from the system.");
	  	int size = q.aortaqp.AQVector.size();
	  	for (int i=0;i<size;i++)
	  	  if (((AQ)(q.aortaqp.AQVector.elementAt(i))).id == q.id)
	  	    {
	  	      q.aortaqp.AQVector.removeElementAt(i);
	  	      break;
	  	    }
	  	
	  }
	
} 