/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * Class for sending an MMS message embedded with a photo to a real cell phone.
 *
 * @see SMSMessageSender
 * @see SendPhoto 
 */ 
public class MMSMessageSender {
	
    Action action;
    
    /** Web message server that forwards the message to the phone. */
    public String serverName = "www.csoft.co.uk";
    /** Port of the message server. */
    public int serverPort = 80;

    String  requestURI = "/sendmms";
    String query = "Username=XXXXXX.XXXXXX&PIN=XXXXXXXX";  // the authorization information required for accessing the message server 

    String requestMethod = "POST";      
    static final String protocol = "HTTP/1.1";    
    static final String contentTypeHeader = "Content-Type: application/x-www-form-urlencoded; charset=\"utf-8\"";
       
    
    /**
     * Construct an MMS message sender.
     * 
     * @param action an action that uses this class to send a photo embedded in a MMS message to a real cell phone
     * @param sendTo the NO. of the phone to send the message to
     * @param message the message sent together with the photo
     * @param WAPPushFileName1 the pathname of the photo to be sent
     * @param WAPPushFileData1 the binary data of the photo to be sent
     */
    public MMSMessageSender (Action action, String sendTo, String message, String WAPPushFileName1, String WAPPushFileData1) 
      {
    	
    	this.action = action;
    	
    	query += "&SendTo=" + sendTo;    	
    	query += "&Message=" + message;    	
    	query += "&WAPPushFileName1=" + WAPPushFileName1;    	
    	query += "&WAPPushFileData1=" + WAPPushFileData1;    	
      } 
      
    
    /** Send the MMS message to the phone through a Web message server. */
    public boolean process() 
      {
    	
    	Socket socket = null;	    
	    int reqId = 0;
	    char space = ' ';	
    	String newLine = "\r\n";    	
    	boolean suc = false;
		
 	    try 
 	      {
 	    	String request = assembleHTTPRequest(); 	    	
 	    	socket = new Socket(serverName,serverPort); 	    	
 	    	InputStream ins = sendRequest(request,socket);		
		    suc = receiveResponse(ins);		
          }  
	     catch (Exception e) {e.printStackTrace();}
	     
	     return suc;	
      } 
    
    
    public String assembleHTTPRequest() 
      {
    	
        char space = ' ';
        String newLine = "\r\n";
        String slash = "/";

    	StringBuffer sb = new StringBuffer(requestMethod);        
        sb.append(space);        
        sb.append(requestURI);

        if (query != null && requestMethod.equalsIgnoreCase("GET")) 
          {
            sb.append('?');
            sb.append(query.trim());
          }

	    sb.append(space);
	    sb.append(protocol);
        sb.append(newLine);

	    if (query != null && requestMethod.equalsIgnoreCase("POST")) 
	      {
	      	sb.append(contentTypeHeader);
	        sb.append(newLine);
	    
	        sb.append("Content-Length: ");
	        sb.append(query.length());	    
	        sb.append(newLine);

	        sb.append(newLine);
	        sb.append(query);	    
          }
          
        sb.append(newLine);       
	
        return sb.toString();
      } 
    
    
    public InputStream sendRequest (String request,Socket socket) 
      {
    	InputStream responseIns = null;
        PrintWriter sender = null;

        try 
          {
            sender = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())));
            responseIns = socket.getInputStream();
            sender.print(request);
            sender.flush();
          }
        catch (Exception e) {e.printStackTrace();}
        
        return responseIns;
      } 
    
    
    public boolean receiveResponse(InputStream ins)  
      {
    	
    	boolean suc = false;    	
    	
        try 
          {
            BufferedReader br = new BufferedReader(new InputStreamReader(ins));

            String line;            
                      
            action.q.logFileWriter.println("#");
            action.q.logFileWriter.println("The HTTP response resulted from sending the message:");
	        while (!(line=br.readLine()).equals("")) 
	          action.q.logFileWriter.println(line); 
	          
	        line = br.readLine().trim();
	        action.q.logFileWriter.println(line);
	        action.q.logFileWriter.println("#");
	        action.q.logFileWriter.flush();
	        
	        if (line.indexOf("OK") != -1)
	          suc = true;
	        else 
	          suc = false;	        
	        
	        br.close();	
	           
	      }
        catch (Exception e) {e.printStackTrace();}
        
        return suc;      
      }     
}  
