/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.util.*;

/** 
 * Abstract class for operators in AQ plan. 
 *
 * @see AQ
 */
public abstract class Operator {
	
	/** Type of this operator. */
	public byte type;	
	public static final byte SELECTION = 1;
	public static final byte SCAN = 2;
	public static final byte JOIN = 3;
	public static final byte ACTION = 4;	
	
	/* Currently we only support query operators with one or two input queues */
	public Vector inputQueue = new Vector(20,20);
    public Vector inputQueue2 = new Vector(10,20);    
    public Vector outputQueue;
	
	/** Upstream operator of this operator. */
	public Operator outputOperator = null;
	
	/** 
	 * The action-embedded query that this operator belongs to. 
	 */
	public AQ q;
	
	
	public Operator () {}
		
	
	/** 
	 * Insert an output tuple to the output queue. 
	 *
	 * @param t the tuple for output	 
	 */
	public void enQueue (Tuple t)
	  {
	    outputQueue.addElement(t);	  	       
	  }	  
	
	
	/**
	 * Extract an input tuple from one of the input queues. 
	 *
	 * @param qno the No. of the input queue to extract a tuple from.
	 *
	 * @return the tuple extracted from the input queue
	 *
	 */
	public Tuple deQueue (int qno)
	  {
	  	Tuple t = null;
	  	
	  	if (qno == 1)
	  	  {
	  	  	t = (Tuple)(inputQueue.elementAt(0));
	  	  	inputQueue.removeElementAt(0);
	  	  }
	  	else 
	  	  if (qno == 2)
	  	    {
	  	    	t = (Tuple)(inputQueue2.elementAt(0));
	  	    	inputQueue2.removeElementAt(0);
	  	    }
	  	  else
	  	   {
	  	     System.err.println("Queue No. Error!");
	  	     System.exit(1);
	  	   }
	  	
	  	return t;
	  }
	  
	 
	 /** Activate this operator. */
	 public void start() {}
	   
	 /** Deactivate this operator. */
	 public void stop() {}
	   
	 /** One-time execution of this operator. */
	 public void getInput() {}
	  
} 