/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * Thread for polling a real cell phone.
 *
 * @see ActionOperator#selectCandidatePhone
 */
public class RealPhonePollingThread extends Thread {
	
	public static final int PHONE_POLLING_TIMEOUT = 2*60*1000;  // 2 minutes (uint: millisecond)
	
	public static final String PHONE_POLLING_MESSAGE = "Someone enters the pervasive lab just now. If you are willing to receive a real-time photo, please use Java Games & Apps -> AortaPhoneManager -> StatusSender to send back the physical status of the phone to the Aorta server.";
	
	public static final int INITIAL_PORT_NO = 9000; 
	/** The port that the socket server corresponding to a real cell phone uses is: INITIAL_PORT_NO + id of the phone. */
	public int port;
				
	ActionOperator actionOp;	
	
	String phone_no;
	
	ServerSocket serverSocket;
	Socket clientSocket;
	InputStream is;
	OutputStream os;
	
	long responseTime;
	int freeMem = -1;
	
	boolean connected = false;
	
	public RealPhonePollingThread(ActionOperator actionOp, int port, String phone_no) 
	  {
		
		this.actionOp = actionOp;
		
		this.port = port;
		
		this.phone_no = phone_no;		
		
		try {serverSocket = new ServerSocket(port);}
	    catch (Exception e) {e.printStackTrace();}	
	  } 
	
	
	/** 
	 * Poll a candidate phone by sending a probing text message to the phone.
	 * Wait at most {@link #PHONE_POLLING_TIMEOUT} for (the user of) the phone to 
	 * send back the current physical status of the phone (e.g., free memory)
	 * through a socket connection.
	 */
	public synchronized void run() 
	  {
		
		try
		  {
		  	SMSMessageSender sms_sender = new SMSMessageSender(actionOp.action, PHONE_POLLING_MESSAGE, phone_no);
	   	    int count = 0;
	   	    boolean suc = false;
	   	    
	   	    responseTime = System.currentTimeMillis();	   	    
	   	   
	   	    while (count < Aorta.RETRY_NUM && suc == false)
	   	      {
	   	          suc = sms_sender.process();
	   	          count ++;
	   	      }   	  	
	   	    if (suc == false) 
	   	      {
	   	      	responseTime = Long.MAX_VALUE;
	   	      	return;
	   	      }
		  	
		  	clientSocket = serverSocket.accept();  // waiting the connection from the client program at the phone side
		  	
		  	is = clientSocket.getInputStream();
		  	os = clientSocket.getOutputStream();
		  	
		  	freeMem = is.read(); 
		  	
		  	connected = true;  // the candidate phone is currently available
		  	
		  	os.write(freeMem);
		  	
		  	responseTime = System.currentTimeMillis() - responseTime;		  	
		  	
		  	if (is != null) is.close();
		  	if (os != null) os.close();
		  	if (clientSocket != null) clientSocket.close();		  	
		  }
		catch (Exception e) {e.printStackTrace();}		
	  } 
} 