/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * Thread for polling a real or simulated robot.
 *
 * @see ER1Simulator
 * @see ActionOperator#selectCandidateRobot
 * @see SchedulingThread
 */

public class RobotPollingThread extends Thread {
	
  	ER1Simulator er1Sim;
  	
  	boolean robotSimulation;
  	
  	boolean connected = false;
  	
  	long pollingTime = Long.MAX_VALUE;
  	
  	RobotPollingThread (ER1Simulator er1Sim, String ip, Aorta aortaqp)
  	  {
  	  	this.er1Sim = er1Sim;
  	  	this.er1Sim.ip = ip;
  	  	
  	  	robotSimulation = aortaqp.ROBOT_SIMULATION; 	  	  	  		  		  	
  	  }
  	  
  	/** 
     * Poll a candidate robot and know the current physical status of the robot. 
     * Wait at most {@link ER1Simulator#POLLING_TIMEOUT_LIMIT} for the robot to response.
     **/
    public void run()
  	  {
  	  	try
  	  	  {
  	  	  	long startTime, endTime;
  	  	  	Socket socket;
            
            startTime = System.currentTimeMillis();
  	  	    
  	  	    if (robotSimulation)
  	  	      socket = new Socket("127.0.0.1",er1Sim.port);  	  	      
  	  	    else
  	  	      socket = new Socket(er1Sim.ip,er1Sim.port);
  	  	      
  	  	    BufferedReader br = new BufferedReader (new InputStreamReader (socket.getInputStream()));
            PrintWriter pw = new PrintWriter (new DataOutputStream(socket.getOutputStream()));
               
            pw.println("position");
            pw.flush();
                             
            String line = br.readLine();  
            line = line.toLowerCase().trim();          
            
            StringTokenizer st = new StringTokenizer(line);

            if (!st.nextToken().equals("ok"))
              return;
              	  
            double x,y,angle;
              	
            x = Double.parseDouble(st.nextToken());
           	// x = x - er1Sim.HOME_LOC_X;         
            y = Double.parseDouble(st.nextToken());
            // y = y - er1Sim.HOME_LOC_Y;
            if (!robotSimulation)
              {
              	x /= 100;
              	y /= 100;
              }           
            angle = Double.parseDouble(st.nextToken());            
           	/* double slopeAngle;     	
            if (er1Sim.loc.x != 0)
     	      slopeAngle = Math.atan(er1Sim.loc.y / er1Sim.loc.x)/Math.PI*180;
            else
     	      slopeAngle = 90;
            if (slopeAngle == 0)
              if (er1Sim.loc.x < 0)
                slopeAngle = 180;
            angle += slopeAngle;
            if (angle > 180)
              angle -= 360;
            if (angle <= -180)
              angle += 360; */                      
            er1Sim.setLoc(x,y,ER1Simulator.HOME_LOC_Z);    	          
     	    er1Sim.setAngle(angle);  
                                  
            br.close();
            pw.close(); 
            
            connected = true; 
            
            endTime = System.currentTimeMillis();
            
            pollingTime = endTime - startTime;             	       
  	  	  }
  	  	catch (Exception e) {e.printStackTrace();} 	  	
  	  }	
}