/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.util.*;

/**
 * Scan operator in AQ plan.
 *
 * @see AQ
 */
public class ScanOperator extends Operator {
	
	String sensorQueryString;
		
	long lastActionExecTime = 0;
	long lastEventArrivalTime = 0;
	/** System-controlled minimun time interval between two action execution, about 5 seconds. */		
	public static long DEFAULT_ACTION_EXEC_INTERVAL = 5*Aorta.DEFAULT_SAMPLE_PERIOD;  
	long actionExecInterval = DEFAULT_ACTION_EXEC_INTERVAL;
	
	public ScanOperator (AQ q)
	  {
	  	type = SCAN;
	  	inputQueue2 = null; // scan operator has only one input queue
	  	  	
	  	this.q = q;
	  	this.sensorQueryString = q.sensorQueryString;	  
	  }
	  
	  
	public void start()
	  {
	  	if (q.aortaqp.SENSOR_SIMULATION)
	  	  {
	  	  	q.simsq = new AortaSimulatedSensorQuery(sensorQueryString,this); 
	  	    q.simsq.start();
	  	  }
	  	else
	  	  q.sq = new AortaSensorQuery(sensorQueryString, this);
	  }
	  
	
	public void stop()
	  {
	  	if (q.aortaqp.SENSOR_SIMULATION)
	  	  q.simsq.stopSimulatedSensorQuery();
	  	else
	  	  q.sq.stopSensorQuery();
	  }  

	
	/** 
	 * Push a device stream tuple from a real or simulated triggering device to its upstream operator.
	 * A prelimenary load shedding mechanism is implemented to prevent unnecessary duplicate action execution
	 * upon a single event.
	 */
	public void getInput()
	  {
	  	Tuple t = deQueue(1);
	  	
	  	if (System.currentTimeMillis() - lastEventArrivalTime >= 2*Aorta.DEFAULT_SAMPLE_PERIOD)
	  	  actionExecInterval = DEFAULT_ACTION_EXEC_INTERVAL;
	  	lastEventArrivalTime = System.currentTimeMillis();	
	  		  	
	  	if (q.SCAN_DUPLICATE_OPTIMIZATION)
	  	  if (System.currentTimeMillis() - lastActionExecTime < actionExecInterval)
	  	    return;
	  	  else
	  	    {
	  	    	lastActionExecTime = System.currentTimeMillis(); 
	  	    	actionExecInterval *= 2;
	  	    } 
	  	
	  	enQueue(t); // output the tuple	  	
	  
	  	outputOperator.getInput(); 	  	  	
	  } 
  
}