/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;

/** 
 * Selection operator in AQ plan.
 *
 * @see AQ 
 */
public class SelOperator extends Operator {
	
	Vector cache;
	
	public SelOperator (AQ q)
	  {
	  	type = SELECTION;
	  	this.q = q;
	  	inputQueue = inputQueue2 = null;
	  }
	  
	public void start()
	  {
	  	if (q.SELECTION_OPTIMIZATION)
	  	  cache = findDeviceIDSelection();
	  	else 
	  	  cache = null;
	  }
	
	public void stop()
	  {
	  	
	  }
	  
	public void getInput()
	  {
	  	Vector v;
	  	if (q.SELECTION_OPTIMIZATION)
	  	  v = cache;
	  	else 
	  	  v = findDeviceIDSelection();
	  	  
	  	Tuple t;
		for (int i=0;i<v.size();i++)  // output
		  { 
		    t = (Tuple)v.elementAt(i);
		    enQueue(t);	
          }              
	  }
	  
	  
	/** 
	 * Given the name and value of a selection attribute, find the corresponding 
	 * candidate devices with the attribute value.
	 *
	 * @return a vector each element of which is a tuple that encapsulates the information 
	 *         required to execute the action about a candidate device 
	 *
	 * @see Tuple
	 */
	public Vector findDeviceIDSelection() 
	  {
		BufferedReader br;
        String did = null;
        String deviceBaseStation = null;
        String str,s;
        int start, stop;
        Vector v = new Vector (5,5);
      
        if (q.selectionAttrN.indexOf(Aorta.SENSOR_SHORT) != -1)
          deviceBaseStation = Aorta.SENSOR_BASESTATION;
        else
          if (q.selectionAttrN.indexOf(Aorta.CAMERA_SHORT) != -1)
            deviceBaseStation = Aorta.CAMERA_BASESTATION;
          else
            if (q.selectionAttrN.indexOf(Aorta.PHONE_SHORT) != -1)
              deviceBaseStation = Aorta.PHONE_BASESTATION;
            else
              if (q.selectionAttrN.indexOf(Aorta.ROBOT_SHORT) != -1)
                deviceBaseStation = Aorta.ROBOT_BASESTATION;            
      
        String selectionAttrN = q.selectionAttrN.substring(2);
        String selectionAttrV = q.selectionAttrV;        
        try 
		  {
		    br = new BufferedReader(new FileReader(deviceBaseStation));
                        
            while ((str = br.readLine()) != null) 
              {
                str = str.toLowerCase().trim();
              
                start = str.indexOf("<model>");
                if (start == -1) continue;
                start += "<model>".length();
                stop = str.indexOf("</model>",start);
                if (stop == -1) continue;
              
                if (this.outputOperator == q.actionOp)
                  if (!str.substring(start,stop).trim().equals(q.action.devInf.model.toLowerCase()))
                    continue;                  
                if (this.outputOperator == q.actionOp2)
                  if (!str.substring(start,stop).trim().equals(q.action2.devInf.model.toLowerCase()))
                    continue;
              
                start = str.indexOf("<" + selectionAttrN + ">");
                if (start == -1) continue;
          
                stop = str.indexOf(">", start);
                start = stop + 1;  
                stop = str.indexOf("</",start);
                s = str.substring(start, stop);
          
                if (!selectionAttrV.equals(s) && !selectionAttrV.equals("$any")) continue;
                else 
                 {
            	   start = str.lastIndexOf("<id>",start);
            	   if (start == -1) 
            	     {
            	       System.err.print("Invalid format of the ");
            	       if (this.outputOperator == q.actionOp)
            	         System.out.print(q.action.devInf.type);
            	       else if (this.outputOperator == q.actionOp2)
            	           System.out.print(q.action2.devInf.type);
            	       System.err.println(" base station.");
	      	           System.exit(1);
            	     }
            	   start += "<id>".length();
            	   stop = str.indexOf("</",start);
            	   did = str.substring(start,stop).trim(); 
            	   Vector vt = new Vector(2,1);
            	   vt.addElement(did);  // the id of the candidate device
            	 
            	   // the authorization information about a camera: username + password
            	   if (deviceBaseStation.equals(Aorta.CAMERA_BASESTATION))
            	     {
            	   	   start = str.indexOf("<username>");
            	       if (start == -1) 
            	        {
            	  	      System.err.println("Invalid format of the camera base station.");
	      	              System.exit(1);
            	        }
            	       stop = str.indexOf("</",start);
            	       vt.addElement(str.substring(start+10,stop).trim());
            	       start = str.indexOf("<password>",start);
            	       if (start == -1) 
            	         {
            	  	       System.err.println("Invalid format of the camera base station.");
	      	               System.exit(1);
            	         }
            	       stop = str.indexOf("</",start);
            	       vt.addElement(str.substring(start+10,stop).trim());            	   	 
            	     }
            	             	 
            	   // the instantiated values of action input parameters (if any)            	 
            	   String actionParams [];
            	   if (this.outputOperator == q.actionOp2)
            	     actionParams = q.actionParams2;
            	   else 
            	     actionParams = q.actionParams;
            	   if (deviceBaseStation == Aorta.ROBOT_BASESTATION)
            	     actionParams[0] = "r.ip";
            	   int i =0;            	 
            	   while (actionParams[i] != null)
            	     {
            	       if (actionParams[i].indexOf(q.selectionAttrN.substring(0,2)) != -1)
            	         {
            	           start = str.indexOf("<" + actionParams[i].substring(2) + ">");
            	           if (start != -1)
            	             {
            	               start = str.indexOf(">",start) + 1;
            	               stop = str.indexOf("</",start);
            	               vt.addElement(str.substring(start,stop).trim());	
            	             }	
            	          }            	      
            	        i++;	
            	      }            	      
            	 
            	   Tuple t = new Tuple(vt);
            	   t.qid = q.id;
            	   t.seqNo = q.aortaqp.seqNo;
            	    
            	   // e.g., the fields of the tuple are: phone_id phone_no
            	   // or                                 sensor_id sensor_loc
            	   // or								 robot_id robot_ip
            	   v.addElement(t);            	             	              	       	 
                 }         
              } 
	 
	        q.aortaqp.seqNo++;
	        
	        br.close(); 
	    
	      } 
	    catch (Exception e) {e.printStackTrace();}	
	    
	    return v;	      
      } 
  
} 