/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;
import java.net.*;
import sun.misc.*;

/**
 * Implementation of the system built-in action sendmsg() 
 * on programmable cell phones.
 *
 * @see ActionOperator
 */
public class SendMsg extends Action {
   
    /** Size limit of a text message sent to a real cell phone. */
    public static final int MESSAGE_SIZE_LIMIT = 200;  // unit: byte
    
    boolean firstTime = true; 
    
    Aorta aortaqp; 
    

	/** 
	 * @param aortaqp the Aorta query processor that this action registers to
	 */
	public SendMsg(Aorta aortaqp) 
	  { 
	    name = "sendmsg";
	    
	    profile = "../action_profiles/system/sendmsg.xml";
	    
	    params[0].name = "phone_no";
	    params[0].type = "String";
	    params[1].name = "message_filename";
	    params[1].type = "String";
	    paramNum = 2;
	    
	    flag = Action.SYSTEM_BUILT_IN;	    	    
	    
	    this.aortaqp = aortaqp;  
	    
	    if (!parseProfile())
	      {
	      	System.err.println("Fail to initialize the system built-in action sendmsg().");
	      	System.exit(0);
	      }	 	     
	  }
	  
	 
	/**
	 * This method implements the operations involved in the action 
	 * It uses the J2ME phone emulator to execute the action if {@link Aorta#PHONE_EMULATION}\
	 * is set to be <code>true</code>;
	 * uses a real cell phone otherwise.
     * <p>
	 * The action has two input parameters: 
	 * <ul>
	 * <li> The number of a phone.
	 * <li> The pathname of the message text file to be sent to the phone.
	 * </ul>
	 *
	 * @param s the string representation of the instantiated values of the action input parameters
	 * @return a {@link Boolean} object. The value of it is <code>true</code> if the action execution is successful;
	 *         <code>false</code> otherwise.
	 *
	 * @see #doActionRealPhone
	 * @see #doActionJ2MEPhoneEmulator
	 */
	public Object doAction(String [] s) 
	  {		
		if (aortaqp.PHONE_EMULATION)
		  return doActionJ2MEPhoneEmulator(s);
		else
		  return doActionRealPhone(s);		
      } 
    
	
	public Object doActionJ2MEPhoneEmulator(String [] s) 
	  {
		 try 
	       {
	          String messageFileName = s[1];
	          BufferedReader br = new BufferedReader(new FileReader(messageFileName));
	          String str, messageText = "";
	          while ( (str = br.readLine()) != null) 
	            messageText += str + "\r\n";
	       
	          if (aortaqp.firstTime)
	            {
                  aortaqp.ws = new WebServer (aortaqp.serverPort,aortaqp.MESSAGE_FILE,aortaqp.PHOTO_FILE);
                  FileWriter mf = new FileWriter(aortaqp.MESSAGE_FILE);
                  mf.write("-1",0,2); 
                  mf.close(); 
                  aortaqp.ws.send();
                  aortaqp.firstTime = false;
                }
               
              str = String.valueOf(aortaqp.messageId++) + messageText;
	          PrintWriter pw = new PrintWriter (new BufferedWriter (new OutputStreamWriter(new FileOutputStream(aortaqp.MESSAGE_FILE))));
	          pw.println(str);
	          pw.flush();
	          pw.close();	
	     
	          aortaqp.ws.send();	      
	     
	          System.out.println("exec action sendmsg(" + s[0] + "," + "\"" + s[1] + "\")"); 
	          if (q.type == aortaqp.EVENT_DRIVEN_QUERY)
                System.out.print("Aorta>");
              if (q.type == aortaqp.TRADITIONAL_CONTINUOUS_QUERY)
                if (!firstTime)
                  System.out.print("Aorta>");
                else 
                  {
                  	System.out.println();
                  	firstTime = false;
                  } 
         	          	     
	       } 
	   catch (Exception e) {e.printStackTrace(); return new Boolean("false");}
	   
	   return new Boolean("true");	   	
	} 
	
	
	public Object doActionRealPhone(String [] s)  
	  {
	     try
	       {
	    	  String messageFileName = s[1];
	          BufferedReader br = new BufferedReader(new FileReader(messageFileName));
	          String str, messageText = "";
	          while ( (str = br.readLine()) != null) 
	            messageText += str + "\r\n";
	     
	          SMSMessageSender smsms = new SMSMessageSender(this,messageText,s[0]);
	          boolean suc = smsms.process();
	      
	          System.out.print("exec ");
	          System.out.println("action sendmsg(" + s[0] + "," + "\"" + s[1] + "\")"); 
	          System.out.print("Aorta>");
	      
	          if (suc == false)
	            {
	              System.err.println("Fail to execute the action. Please check whether the phone " + s[0] + " has been turned on and it is working properly.");
	              return new Boolean("false");
	            }	          	      
	       } 
	     catch (Exception e) {e.printStackTrace(); return new Boolean("false");} 
	     
	     return new Boolean("true"); 
	} 
		
}