/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;
import java.net.*;
import sun.misc.*;

/**
 * Implementation of the system built-in action sendphoto() on programmable cell phones with MMS support.
 *
 * @see ActionOperator
 */
public class SendPhoto extends Action {

      /** Message sent to a J2ME phone emulator. */
      public static final String emulation_message = "\r\nA new photo is received.\r\n" + /*"Alarm!\r\n" +
                                 "There is something abnormal in the pervasive lab now.\r\n" +
                                 "Please check as soon as possible.\r\n" +*/
                                 "<Press the \"View Photo\" button to see the photo>";
      
      /** Message sent to a real cell phone. */
      public static final String real_message = "Notice! Someone enters the pervasive lab just now.";
           
      /** Size limit of a photo sent to a real cell phone. */
      public static final int PHOTO_SIZE_LIMIT = 8600;  // unit: byte
      
      boolean firstTime = true;  
      
      Aorta aortaqp; 
               
           
    /** 
	 * @param aortaqp the Aorta query processor that this action registers to
	 */
	 public SendPhoto (Aorta aortaqp) 
        { 
          name = "sendphoto"; 
          
          profile = "../action_profiles/system/sendphoto.xml";
          
          params[0].name = "phone_no";
	      params[0].type = "String";
	      params[1].name = "photo_filename";
	      params[1].type = "String";
	      paramNum = 2;
	  
	      flag = Action.SYSTEM_BUILT_IN;	  
          
          this.aortaqp = aortaqp; 
          
          if (!parseProfile())
	        {
	      	  System.err.println("Fail to initialize the system built-in action sendphoto().");
	          System.exit(0);
	        }             
        }
	
	
	  /**
	   * This method implements the operations involved in the action. 
	   * It uses the J2ME phone emulator to execute the action if {@link Aorta#PHONE_EMULATION} 
	   * is set to be <code>true</code>;
	   * uses a real cell phone otherwise.
	   * <p>
	   * This action has two input parameters: 
	   * <ul>
	   * <li> The number of a phone.
	   * <li> The file (path) name of the photo image file to be sent to the phone.
	   * </ul>
	   *
	   * @param s the string representation of the instantiated values of the action input parameters
	   * @return a {@link Boolean} object. The value of it is <code>true</code> if the action execution is successful;
	   *         <code>false</code> otherwise.
	   *
	   * @see #doActionRealPhone
	   * @see #doActionJ2MEPhoneEmulator
	   */
	  public Object doAction(String [] s) 
	    {		
		   if (aortaqp.PHONE_EMULATION)
		     return doActionJ2MEPhoneEmulator(s);
		   else
		     return doActionRealPhone(s);		
        }  
    
	
	  public Object doActionJ2MEPhoneEmulator (String [] s) 
	    {
	       try 
	         {
	           String imageFileName = s[1];
               
               if (aortaqp.firstTime)
	             {
                   aortaqp.ws = new WebServer (aortaqp.serverPort,aortaqp.MESSAGE_FILE,aortaqp.PHOTO_FILE);
                   FileWriter mf = new FileWriter(aortaqp.MESSAGE_FILE);
                   mf.write("-1",0,2); 
                   mf.close(); 
                   aortaqp.ws.send();
                   aortaqp.firstTime = false;
                 }
            
	     
	           String str = String.valueOf(aortaqp.messageId++) + "   " + emulation_message;
	           PrintWriter pw = new PrintWriter (new BufferedWriter (new OutputStreamWriter (new FileOutputStream(aortaqp.MESSAGE_FILE))));
	           pw.println(str);
	           pw.flush();
	           pw.close();	     
	     
	           Runtime.getRuntime().exec("cp " + imageFileName + " " + aortaqp.PHOTO_FILE);
	           
	           aortaqp.ws.send();	     
	     	  
	           System.out.println("exec action sendphoto(" + s[0] + "," + "\"" + imageFileName + ")"); 
	           if (q.type == aortaqp.EVENT_DRIVEN_QUERY)
                System.out.print("Aorta>");
              if (q.type == aortaqp.TRADITIONAL_CONTINUOUS_QUERY)
                if (!firstTime)
                  System.out.print("Aorta>");                   
                else 
                  {
                  	System.out.println();
                  	firstTime = false;
                  } 	     
	   } 
	   catch (Exception e) {e.printStackTrace(); return new Boolean("false");}
	   
	   return new Boolean("true");	   	
	} 
	
	
	public Object doActionRealPhone(String [] s)  
	  {
	     try
	       {
	          String imageFileName = s[1];
	          String str;          
	          InputStreamReader ir = new InputStreamReader (new FileInputStream(imageFileName),"ISO-8859-1");
 	          BufferedReader br = new BufferedReader (ir);
 	          int c;
 	          String data = "";
  		      while ( (c=br.read()) != -1)
  		        {
  		          str = Integer.toHexString(c);
  		      
  		          if (str.length() == 1)
  		      	    data += "0";
  		      	  data += str;
  		        }
  		  
  		      MMSMessageSender mmsms = new MMSMessageSender(this,s[0], real_message, "alarm.jpg", data);
	          boolean suc = mmsms.process();
	      
	          System.out.print("exec ");
	          System.out.println("action sendphoto(" + s[0] + "," + "\"" + imageFileName.substring(0,imageFileName.indexOf("_half")) + ".jpg\"" + ")"); 
	          System.out.print("Aorta>");
	      
	          if (suc == false)
	            {
	              System.err.println("Fail to execute the action. Please check whether the phone " + s[0] + " has been turned on and is working properly.");
	      	      System.out.print("Aorta>");
	      	      return new Boolean("false");
      	        }  
	      
	       } 
	     catch (Exception e) {e.printStackTrace(); return new Boolean("false");}
	    
		 return new Boolean("true");
	} 
		
}