/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/**
 * Thread class that continuously monitors the START clause of an AQ.
 *
 * @see AQ
 * @see TimerFunctions
 */
public class StartClause extends Thread {
	
    /** The AQ that this START clause belongs to. */
    public AQ q;
    
    /** @param q the AQ that this START clause belongs to. */
    public StartClause (AQ q)
      {
        this.q = q;     	  
      }    
    
    /**
     * Based on the specification of the timer function or variable in the START clause, 
     * dynamically tune a repetitive timer that triggers the start of {@link #q}. 
     */
    public synchronized void run() {
    	
    	int year = 0,month = 0,day = 0;
    	int hour = 0,minute = 0,second = 0;
    	int year2 =0, month2 = 0, day2 = 0;
    	int hour2 = 0, minute2 = 0, second2 = 0;
    	String date,date2;
    	
    	try {
    	while (true)
    	  {
    	  	if (q.queryDropped)
    	  	  return;
    	  	
    	  	if (q.startTimerFuncName.equals(TimerFunctions.$NOW))
    	  	  q.start();
             
            if (q.startTimerFuncName.equals(TimerFunctions.ATTIME))
              {
              	hour = Integer.parseInt(q.startTimerFuncParams[0]);
              	minute = Integer.parseInt(q.startTimerFuncParams[1]);
              	second = Integer.parseInt(q.startTimerFuncParams[2]);
              	
              	if (TimerFunctions.atTime(hour,minute,second))
              	  q.start();
              	else 
              	  {
                     if (q.START_OPTIMIZATION)
                       {
                      	 long sleep_interval = TimerFunctions.computeTimeInterval(hour,minute,second);  
                         Thread.currentThread().sleep(sleep_interval);
                         q.start();
                       }
                     else
                       {
                       	 do {} while (!TimerFunctions.atTime(hour,minute,second));
                       	 q.start();
                       }             	  	
              	  }      	               	  
              } 
              
              
            if (q.startTimerFuncName.equals(TimerFunctions.ONDAY))
              {
              	year = Integer.parseInt(q.startTimerFuncParams[0]);
              	month = Integer.parseInt(q.startTimerFuncParams[1]);
              	day = Integer.parseInt(q.startTimerFuncParams[2]);
                
              	if (TimerFunctions.onDay(year,month,day))
              	  q.start();
              	else 
              	  {
                     if (q.START_OPTIMIZATION)
                       {
                       	 long sleep_interval = TimerFunctions.computeDayTimeInterval(year,month,day);  
                         if (sleep_interval == -1) // the specified day has already passed
                           break;
                         Thread.currentThread().sleep(sleep_interval);
                         q.start();
                       }
                     else 
                       {
                       	 do {} while (!TimerFunctions.onDay(year,month,day));
                       	 q.start();
                       }            	  	
              	  }      	     
              } 	  	  
    	  	
    	  	if (q.startTimerFuncName.equals(TimerFunctions.INTIMEINTERVAL))
    	  	  {
    	  	  	hour = Integer.parseInt(q.startTimerFuncParams[0]);
              	minute = Integer.parseInt(q.startTimerFuncParams[1]);
              	second = Integer.parseInt(q.startTimerFuncParams[2]);
              	
              	hour2 = Integer.parseInt(q.startTimerFuncParams[3]);
              	minute2 = Integer.parseInt(q.startTimerFuncParams[4]);
              	second2 = Integer.parseInt(q.startTimerFuncParams[5]); 
              	
              	if (TimerFunctions.inTimeInterval(hour,minute,second,hour2,minute2,second2) )
    	  	      q.start();
    	  	    else 
    	  	      {
    	  	      	if (q.START_OPTIMIZATION)
    	  	      	  {
    	  	      	     long sleep_interval = TimerFunctions.computeTimeInterval(hour,minute,second);  
                         Thread.currentThread().sleep(sleep_interval);
                         q.start(); 
    	  	      	  }
    	  	      	else 
    	  	      	  {
    	  	      	  	do {} while(!TimerFunctions.inTimeInterval(hour,minute,second,hour2,minute2,second2));
    	  	      	  	q.start();
    	  	      	  }
    	  	      }
    	  	  }   
    	  	
    	  	
    	  	if (q.startTimerFuncName.equals(TimerFunctions.INDAYINTERVAL))
    	  	  {
    	  	    year = Integer.parseInt(q.startTimerFuncParams[0]);
                month = Integer.parseInt(q.startTimerFuncParams[1]);
              	day = Integer.parseInt(q.startTimerFuncParams[2]);
              	
              	year2 = Integer.parseInt(q.startTimerFuncParams[3]);
              	month2 = Integer.parseInt(q.startTimerFuncParams[4]);
              	day2 = Integer.parseInt(q.startTimerFuncParams[5]); 
              	
              	if (TimerFunctions.inDayInterval(year,month,day,year2,month2,day2))
    	  	      q.start();
    	  	    else 
    	  	      {
    	  	      	if (q.START_OPTIMIZATION)
    	  	      	  {
    	  	      	     long sleep_interval = TimerFunctions.computeDayTimeInterval(year,month,day);  
                         if (sleep_interval == -1)
                           break;               
                         Thread.currentThread().sleep(sleep_interval);
                         q.start(); 
    	  	      	  }
    	  	      	else 
    	  	      	  {
    	  	      	  	do {} while(!TimerFunctions.inDayInterval(year,month,day,year2,month2,day2));
    	  	      	  	q.start();
    	  	      	  }
    	  	      }	
    	  	  }
    	  	  
    	  	
    	  	if (q.startTimerFuncName.equals(TimerFunctions.ONDAYOFWEEK))
    	  	  {
    	  	     date = q.startTimerFuncParams[0];
    	  	         	  	     
    	  	     if (TimerFunctions.onDayOfWeek(date))
    	  	       q.start();
    	  	     else 
    	  	       {
    	  	       	  if (q.START_OPTIMIZATION)
    	  	      	  {
    	  	      	     long sleep_interval = TimerFunctions.computeDayOfWeekTimeInterval(date);
                         Thread.currentThread().sleep(sleep_interval);
                         q.start(); 
    	  	      	  }
    	  	      	  else 
    	  	      	    {
    	  	      	  	  do {} while(!TimerFunctions.onDayOfWeek(date));
    	  	      	  	  q.start();
    	  	      	    }
    	  	       }	
    	  	  }
    	  	  
    	  	if (q.startTimerFuncName.equals(TimerFunctions.INDAYOFWEEKINTERVAL))
    	  	  {
    	  	    date = q.startTimerFuncParams[0];
    	  	    date2 = q.startTimerFuncParams[1];
    	  	        	  	    
    	  	    if (TimerFunctions.inDayOfWeekInterval(date,date2))
    	  	      q.start();
    	  	     else 
    	  	       {
    	  	       	  if (q.START_OPTIMIZATION)
    	  	      	  {
    	  	      	     long sleep_interval = TimerFunctions.computeDayOfWeekTimeInterval(date);
                         Thread.currentThread().sleep(sleep_interval);
                         q.start(); 
    	  	      	  }
    	  	      	  else 
    	  	      	    {
    	  	      	  	  do {} while(!TimerFunctions.inDayOfWeekInterval(date,date2));
    	  	      	  	  q.start();
    	  	      	    }
    	  	       }		
    	  	  }
    	  	    
    	  	notifyAll();
    	  	wait();
    	  	  
    	  } 
         } 
    	 catch (Exception e) {e.printStackTrace();}
    	  
    }    
   
} 