/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.applet.*;
import com.sun.image.codec.jpeg.*;
import java.awt.geom.GeneralPath;

/** 
 * Create an {@link Image} from a {@link URL} or a disk image file
 * and draw it in a {@link Frame}.
 *
 * @author Hejun Wu (whjnn@cs.ust.hk)
 *
 * @see Photo 
 */
public class TakePhotos extends Frame {
	
   	Image image;
   	
   	ImageCanvas canva;
   	
   	boolean firstTime;
   	
   	Aorta aortaqp;
   	
   	   	
    public TakePhotos(int width, int height, Aorta a) 
      {
    	firstTime = true;
    	
        aortaqp = a;        
    
        setSize(width,height);
        
        addWindowListener(new WindowAdapter() 
          {
             // exit the system when closing the photo display window
             public void windowClosing(WindowEvent e) 
               {
                 System.out.println("exit");
                 System.out.println("Thanks for using Aorta. Goodbye!");
                 for (int i=0; i<aortaqp.AQVector.size();i++)
                   ((AQ)aortaqp.AQVector.elementAt(i)).destroy();
                 System.exit(0);
               }
          });      
        
      } 
          
    
    public Image takeCurrentPhoto(URLConnection uc)
      {
    	
    	if(!firstTime) image.flush();    	
   	   
        try {image = createImage((sun.awt.image.URLImageSource)uc.getContent());} 
		catch (Exception e) {e.printStackTrace(); return null;}
        
        if (image == null)
          return null;
          
        createCanva();
        
        return image;   	
      }
      
   
    public void createCanva ()
      {
      	if(firstTime)
          canva=new ImageCanvas(image);
        
        canva.setCanvaImage(image);
      }
      
    
    public Image getCurrentPhoto()
      {
    	return image;
      }
    
    
    public void saveCurrent(String givename)
      {
    	canva.saveToDisk(givename);
      }
    
    
    public boolean drawCurrentPhoto()
      {
    	try 
    	  {
    	  	if(firstTime)
    	      {
    		    firstTime = false;
    		    add("Center", canva);
    	      }
    	    else
    	      {
    		    remove(canva);
	   	        add("Center", canva);
	   	        canva.repaint();
	          }
    	    show(); 
          }  
        catch (Exception e) {e.printStackTrace(); return false;}
        
        return true;         	        
     } 

} 


class ImageCanvas extends Component {
    
    Image image;
    
    BufferedImage bimage;
    
    String filename;
    
    boolean imagechanged;
    
    Graphics2D g2d;
    
    ImageObserverSpy spy = new ImageObserverSpy(this);
	
    
    ImageCanvas(Image image) {}  
    
    public void setCanvaImage(Image paraimage)
      {
    	this.image=paraimage;
        image.getWidth(spy);
        image.getHeight(spy);
        image.getProperty("test", spy);
        bimage=toBufferedImage(image);
      }
	
	
	public int getImageWidth() 
	  {
		return this.image.getWidth(spy);       
      }
	
	
	public int getImageHeight()
	  {
		return this.image.getHeight(spy);
	  }
	
	
    public void paint(Graphics g ) 
      {
    	   		
        g.drawImage(image, 0, 0, spy);        
      }
    
    
    public static boolean hasAlpha(Image image) 
      {
     	// if buffered image, the color model is readily available
        if (image instanceof BufferedImage) 
          {
            BufferedImage bimage = (BufferedImage)image;
            return bimage.getColorModel().hasAlpha();
          }
    
        // use a pixel grabber to retrieve the image's color model, grabbing a single pixel is usually sufficient
        PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
        try {pg.grabPixels();} 
        catch (InterruptedException e) {e.printStackTrace();}
    
        // get the image's color model
        ColorModel cm = pg.getColorModel();
        return cm.hasAlpha();
      }
	
	
    /** This method returns a buffered image with the contents of an image. */
    public static BufferedImage toBufferedImage(Image image) 
      {
        if (image instanceof BufferedImage) 
            return (BufferedImage)image;
           
        // this code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();
    
        /* Determine if the image has transparent pixels; for this method's implementation, 
           see e665 Determining if an Image Has Transparent Pixels */
        boolean hasAlpha = false;
    
        // create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try 
          {
            // determine the type of transparency of the new buffered image
            int transparency = Transparency.OPAQUE;
            if (hasAlpha) transparency = Transparency.BITMASK;        
    
            // create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(image.getWidth(null), image.getHeight(null), transparency);
          } 
        catch (HeadlessException e) 
          { 
            // the system does not have a screen
            e.printStackTrace();
          }
    
        if (bimage == null) 
          {
            // create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
            if (hasAlpha) type = BufferedImage.TYPE_INT_ARGB;
            
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
          }
    
        // copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;
    }
    
    
	public static void saveToFile(BufferedImage bi,Graphics2D g2,String filename)
	  {
		int w,  h, hh;
		
		w=bi.getWidth();
		h=bi.getHeight();
		hh=h/2;
        Graphics2D big = bi.createGraphics();

        // use rendering hints from J2DCanvas ..
        big.setRenderingHints(g2.getRenderingHints());
        BufferedImage bi1 = null;

        try 
          {
            // to write the jpeg to a file uncomment the File* lines and comment out the ByteArray*Stream lines.
            File file = new File(filename);
            FileOutputStream out = new FileOutputStream(file);

            JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
            JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(bi);
            param.setQuality(1.0f, false);
            encoder.setJPEGEncodeParam(param);
            encoder.encode(bi);

            FileInputStream in = new FileInputStream(file);
            
            JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
            bi1 = decoder.decodeAsBufferedImage();
          } 
        catch (Exception ex) 
          {
            g2.setColor(Color.red);
            g2.drawString("write permissions on images/test.jpg?", 5, hh*2-5);
            return;
          }

        if (bi1 == null) 
          {
            g2.setColor(Color.red);
            g2.drawString("decodeAsBufferedImage=null", 5, hh*2-5);
            return;
          }

        BufferedImage bi2 = new BufferedImage(bi1.getWidth(),bi1.getHeight(),bi1.getType());
        DataBuffer db1 = bi1.getRaster().getDataBuffer();
        DataBuffer db2 = bi2.getRaster().getDataBuffer();

        for (int i = db1.getSize()-1, j = 0; i >= 0; --i, j++) 
          db2.setElem(j, db1.getElem(i));       
        
    }
    
    
    public void saveToDisk(String givename)
      {
        g2d=bimage.createGraphics();
        saveToFile(bimage, g2d,givename);
      }

}


class ImageObserverSpy implements ImageObserver 
  {

    ImageObserver obs;


    ImageObserverSpy(ImageObserver obs) 
      {
        this.obs = obs;
      }
    

    public boolean imageUpdate(Image img, int infoflags,int x, int y, int width, int height) 
      {
         return obs.imageUpdate(img, infoflags, x, y, width, height);
      }
  }

