/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * Implementation of the system built-in action warn() 
 * on ER1 programmable robots.
 *
 * @see ActionOperator
 */
 
public class Warn extends Action {
 	
 	Aorta aortaqp;
 	
 	ER1Simulator er1Sim [];
 	
 	Warn (Aorta aortaqp)
 	  {
 	  	this.aortaqp = aortaqp;
 	  	
 	  	name = "warn";  
	    
	    profile = "../action_profiles/system/warn.xml";
	    
	    params[0].name = "robot_id";
	    params[0].type = "int";
	    params[1].name = "location";
	    params[1].type = "Point";
	    params[2].name = "text_filename";
	    params[2].type = "String";
	    paramNum = 3;
	    
	    flag = Action.SYSTEM_BUILT_IN;
	    
	    if (!parseProfile())
	      {
	      	System.err.println("Fail to initialize the system built-in action warn().");
	      	System.exit(0);
	      }     
 	  }
 	  
 	  
 	/**
	 * This action implements the operations involved in the action.  
	 * It uses a simulated robot to execute the action if {@link Aorta#ROBOT_SIMULATION} 
	 * is set to be <code>true</code>;
	 * uses a real robot otherwise.
	 * 
	 * <p>
	 * The action has three input parameters:
	 * <ul>
	 * <li> The id of the robot.
	 * <li> The target location where the robot should move to.
	 * <li> The path name of the text file that contains the English phrase to be played by the robot.
	 * </ul>
	 *
	 * @param s the string representation of the instantiated values of the action input parameters
	 * @return a {@link Boolean} object. The value of it is <code>true</code> if the action execution is successful;
	 *         <code>false</code> otherwise. 
	 */
	public Object doAction(String [] s) 
	 {
	   Point loc = Point.parsePoint(s[1]);
	   loc.z = ER1Simulator.HOME_LOC_Z;
	   String sid = s[4];
	   if (sid.equals("1"))  // Sensor 1
	     {loc.x = 0; loc.y = 1.2;}
	   else
	     if (sid.equals("2"))  // Sensor 2
	       {loc.x = -1.3; loc.y = 10;}
	     else
	       if (sid.equals("3") || sid.equals("6"))  // Sensor 3 or Sensor 6
	         {loc.x = 0; loc.y = 5.2;}
	       else
	         if (sid.equals("4") || sid.equals("7"))  // Sensor 4 or Sensor 7
	           {loc.x = 0; loc.y = 1.3;}
	         else
	           if (sid.equals("5") || sid.equals("8"))  // Sensor 5 or Sensor 8
	             {loc.x = -3.3; loc.y = 4.8;}
	           else
	             if (sid.equals("9"))  // Sensor 9
	               {loc.x = -3.5; loc.y = 1.5;}
	             else
	               if (sid.equals("10"))  // Sensor 10
	                 {loc.x = -0.5; loc.y = 6.5;}
	   
	   String messageFile = s[2];
	   String phrase = "";
	   String line;
	   try
	     {
	     	BufferedReader br = new BufferedReader (new FileReader (messageFile));
	     	while ((line = br.readLine())!=null)
	     	  phrase += line;
	     	br.close();
	     }
	   catch (Exception e) {e.printStackTrace();} 	                 
	   	   
	   int id = Integer.parseInt(s[0]);
	   String ip = s[3];
	   try
	     {
	     	Socket socket;
	     	if (aortaqp.ROBOT_SIMULATION)
	     	  socket = new Socket("127.0.0.1",er1Sim[id-1].port);
	     	else
	     	  socket = new Socket(ip,ER1Simulator.initialPort);	        
	     	
	     	BufferedReader readFromRobot = new BufferedReader (new InputStreamReader(socket.getInputStream()));
	     	PrintWriter writeToRobot = new PrintWriter (socket.getOutputStream());
	     	
	     	if (!aortaqp.ROBOT_SIMULATION)
	     	  {
	     	  	writeToRobot.println("set v 50");
	     	  	writeToRobot.println("set w 30");
	     	  	writeToRobot.flush();
	     	  	
	     	  	line = readFromRobot.readLine().toLowerCase().trim();
	     	  	if (!line.equals("ok"))
	     	  	  return new Boolean ("false");
	     	  	line = readFromRobot.readLine().toLowerCase().trim();
	     	  	if (!line.equals("ok"))
	     	  	  return new Boolean ("false");	     	  	
	     	  }
	     	
	     	double currentX = er1Sim[id-1].loc.x;
	     	double currentY = er1Sim[id-1].loc.y;
	     	double currentAngle = er1Sim[id-1].angle;
	     	double targetX = loc.x;
	     	double targetY = loc.y;
	     	
	     	if ((currentX == targetX) && (currentY == targetY))
	     	  {}  // no need to move
	     	else
	     	  {
	     	  	double deltaAngle;
	     	  	if (currentX == targetX)
	     	  	  if (currentY < targetY)
	     	  	    deltaAngle = 90;
	     	  	  else
	     	  	    deltaAngle = -90;
	     	  	else
	     	  	  {
	     	  	  	deltaAngle = Math.atan((targetY - currentY)/(targetX-currentX))/Math.PI*180;
	     	  	  	
	     	  	  	if ((deltaAngle > 0) && (targetY < currentY))
	     	  	  	  deltaAngle -= 180;
	     	  	  	
	     	  	  	if ((deltaAngle < 0) && (targetY > currentY))
	     	  	  	  deltaAngle += 180;
	     	  	  } 
	     	  	deltaAngle -= currentAngle;
	     	  	if (deltaAngle > 180)
	     	  	  deltaAngle -= 360;
	     	  	if (deltaAngle <= -180)
	     	  	  deltaAngle += 360;    	  	

	     	  	writeToRobot.println("move " + (long)deltaAngle + " d");
	     	  	writeToRobot.println("events");
	     	  	writeToRobot.flush();	     	  	
	     	  	readFromRobot.readLine().toLowerCase().trim();
	     	  	if (!aortaqp.ROBOT_SIMULATION)
	     	  	  Thread.currentThread().sleep((long)(deltaAngle*ER1Simulator.ROTATEPERDEG_ESTIMATED_COST));
	     	  	line = readFromRobot.readLine().toLowerCase().trim();	     	  	
	     	  	while (!line.equals("move done"))
	     	  	  {
	     	  	  	if (line.indexOf("move fail") != -1)
	     	  	  	  {
	     	  	  	  	System.err.println("Fail to exec action warn(" + s[0] + "," + s[1] + ",\"" + s[2] + "\"). Fail to rotate the robot towards the target location.");
	                    System.out.print("Aorta>");
	     	  	  	  	return new Boolean(false);
	     	  	  	  }	
	     	  	  	writeToRobot.println("events");
	     	  	  	writeToRobot.flush();
	     	  	  	line = readFromRobot.readLine().toLowerCase().trim();	     	  	  	     	  	  	
	     	  	  } 
	     	    
	     	  	double deltaDistance = Math.sqrt(Math.pow(targetX-currentX,2) + Math.pow(targetY-currentY,2));
	     	  	writeToRobot.println("move " + (long)deltaDistance*100 + " c"); 
	     	  	writeToRobot.println("events");
	     	  	writeToRobot.flush();
	     	  	readFromRobot.readLine().toLowerCase().trim();
	     	  	if (!aortaqp.ROBOT_SIMULATION)
	     	  	  Thread.currentThread().sleep((long)(deltaDistance*ER1Simulator.MOVEPERCM_ESTIMATED_COST));
	     	  	line = readFromRobot.readLine().toLowerCase().trim();	     	  	
	     	  	while (!line.equals("move done"))
	     	  	  {
	     	  	  	if (line.indexOf("move fail") != -1)
	     	  	  	  {
	     	  	  	  	System.err.println("Fail to exec action warn(" + s[0] + "," + s[1] + ",\"" + s[2] + "\"). Fail to drive the robot to the target location.");
	                    System.out.print("Aorta>");
	     	  	  	  	return new Boolean(false);
	     	  	  	  }
	     	  	  	writeToRobot.println("events");
	     	  	  	writeToRobot.flush();
	     	  	  	line = readFromRobot.readLine().toLowerCase().trim();	     	  	  	
	     	  	  }
	     	  	
	     	    writeToRobot.println("play phrase " + "\"" + phrase + "\"");
	     	    writeToRobot.println("events");
	     	    writeToRobot.flush();
	     	    readFromRobot.readLine().toLowerCase().trim();
	     	    if (!aortaqp.ROBOT_SIMULATION)
	     	      Thread.currentThread().sleep((long)ER1Simulator.DEFAULT_SOUND_COST);
	     	  	line = readFromRobot.readLine().toLowerCase().trim();
	     	  	while (!line.equals("play done"))
	     	  	  {
	     	  	  	writeToRobot.println("events");
	     	  	  	writeToRobot.flush();
	     	  	  	line = readFromRobot.readLine().toLowerCase().trim();	     	  	  	
	     	  	  } 
	     	  	
	     	  	er1Sim[id-1].setLoc(targetX, targetY, ER1Simulator.HOME_LOC_Z);
	            deltaAngle += currentAngle;
	            if (deltaAngle <= -180)
	              deltaAngle += 360;
	            if (deltaAngle > 180)
	              deltaAngle -= 360;
	            er1Sim[id-1].setAngle(deltaAngle);
	            
	            if (aortaqp.ROBOT_SIMULATION)
	              {
	              	writeToRobot.println("quit");
	              	writeToRobot.flush();
	              }
	            readFromRobot.close();
	            writeToRobot.close();
	            socket.close();	       	  	       	  	
	     	  }
	     }
	   catch (Exception e) {e.printStackTrace();}
	   
	   System.out.println("exec action warn(" + s[0] + "," + s[1] + ",\"" + s[2] + "\")");
	   System.out.print("Aorta>");
	         
	   return new Boolean("true");	
	 }
 	
}