/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/ 

import java.io.*;
import java.util.*;
import java.net.*;
import sun.misc.*;

/**
 * WebServer that interacts with the J2ME phone emulator.   
 *
 * @author Hejun Wu (whjnn@ust.hk)
 * 
 * @see SendMsg
 * @see SendPhoto
 */
public class WebServer {
	
	String messageFile;
	String photoFile;
	String requestMessageLine;
	String infoUsrPwd;
	String infoServer;
	ServerSocket listenSocket;
	
	/** 
	 * Start the web server. 
	 *
	 * @param port the port of the web server
	 * @param messageFileName the pathname of the text message file to be sent to the J2ME phone emulator
	 * @param photoFileName the pathname of the photo file to be sent to the emulator
	 */
	public WebServer (int port, String messageFileName, String photoFileName)
	  {
	    messageFile = messageFileName;
	    photoFile = photoFileName;
	    infoServer = "HTTP/1.1 401 Authorization Required\nWWW-Authentication: Basic realm =\"name\" ";
	    try {listenSocket = new ServerSocket(port);}
	    catch (Exception e) {e.printStackTrace();}
	  }
		
	
	/** 
	 * Accept connection from the J2ME phone emulator (client) and 
	 * send the required text message and photo to the client side.
	 */
	public void send()
	  {
	    BufferedReader inFromClinet;
	    DataOutputStream outToClient;
	    StringTokenizer tokenizedLine;	
			
		try 
		  {
	        Socket connectionSocket = listenSocket.accept();
	       	
	        inFromClinet = new BufferedReader(new InputStreamReader(connectionSocket.getInputStream()));
		    outToClient = new DataOutputStream(connectionSocket.getOutputStream());
	
		    requestMessageLine = inFromClinet.readLine();
		    if (requestMessageLine == null) 
		      {
		         connectionSocket.close();
		         return;
		      }
		    tokenizedLine = new StringTokenizer(requestMessageLine);
		    if(tokenizedLine.nextToken().equals("GET"))
		      {
		        File localfile;
			    FileInputStream inFile;
			    String docType=tokenizedLine.nextToken();
			
		    	if(docType.equals("/TXT"))
				  localfile=new File(messageFile);
			    else
				  localfile=new File(photoFile);
			    int numofBytes=(int) localfile.length();
		    	if(docType.equals("/TXT"))
				  inFile =new FileInputStream(messageFile);
			    else
				  inFile =new FileInputStream(photoFile);
			    byte[]fileInBytes = new byte[numofBytes];
		
			    inFile.read(fileInBytes);
		
		    	outToClient.writeBytes("HTTP/1.1 200 Document Follows\r\n");
		    	if(docType.equals("/TXT"))
				  outToClient.writeBytes("Content-Type: txt/text\r\n");
			    else
				  outToClient.writeBytes("Content-Type: image/jpeg\r\n");
			    outToClient.writeBytes("Content-Length: " + numofBytes + "\r\n");
		    	outToClient.writeBytes("\r\n");
			    outToClient.write(fileInBytes,0,numofBytes);			
		      }
		    else
		      {
		    	System.out.println("POST");
			    System.out.println(tokenizedLine.nextToken());
		      }	
				
	        connectionSocket.close();       
	      } 
	    catch (Exception e) {e.printStackTrace();}
      }

}