/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/**
 * Thread that continuously monitors the timer function in the WHERE clause of an AQ.
 * This thread will only be created if the value of {@link AQ#WHERE_TIMER_OPTIMIZATION} 
 * is set to be <code>true</code>. 
 *
 * @see AQ
 * @see TimerFunctions
 */
public class WhereTimerQC extends Thread {
	
    /** The AQ that this WHERE timer query constraint belongs to. */
    public AQ q;
  
    /** 
     * Start the corresponding AQ if the current value of this flag is <code>0</code>;
     * stop the AQ if its value is <code>1</code>.
     */
    public byte flag = 0;
    
    /** @param q the AQ that this WHERE timer query constraint belongs to. */
    public WhereTimerQC (AQ q) 
      {
      	this.q = q;
      }     
    
    /**
     * Based on the specification of the timer function in the WHERE clause, 
     * dynamically tune a repetitive timer that triggers the start and stop of 
     * {@link #q} interlacedly. 
    */
    public void run() 
      {
    	int year = 0,month = 0,day = 0;
    	int hour = 0,minute = 0,second = 0;
    	int year2 =0, month2 = 0, day2 = 0;
    	int hour2 = 0, minute2 = 0, second2 = 0;
    	String date,date2;
    	
    	initialize();
    	
    	try {
    		
    	while (true)
    	  {
    	  	
    	  	if (q.queryDropped)
    	  	  return;
    	  	  
    	  	if (q.whereTimerFuncName.equals(TimerFunctions.ATTIME))
              {
              	hour = Integer.parseInt(q.whereTimerFuncParams[0]);
              	minute = Integer.parseInt(q.whereTimerFuncParams[1]);
              	second = Integer.parseInt(q.whereTimerFuncParams[2]);
              	
              	long sleep_interval = TimerFunctions.computeTimeInterval(hour,minute,second);  
                Thread.currentThread().sleep(sleep_interval);
                if (q.start_or_stop) q.start();        	       	               	  
              } 
              
              
            if (q.whereTimerFuncName.equals(TimerFunctions.ONDAY))
              {
              	year = Integer.parseInt(q.whereTimerFuncParams[0]);
              	month = Integer.parseInt(q.whereTimerFuncParams[1]);
              	day = Integer.parseInt(q.whereTimerFuncParams[2]);
              	
              	if (flag == 0)
              	  {
              	  	long sleep_interval = TimerFunctions.computeDayTimeInterval(hour,minute,second);  
                    if (sleep_interval == -1) // the specified day has already passed
                      q.stop();
                    else 
                      {
                  	    Thread.currentThread().sleep(sleep_interval);
                        if (q.start_or_stop) q.start();
                        flag = 1;
                      }
                  }
                else
                  {
                  	Thread.currentThread().sleep(TimerFunctions.ONE_DAY_SECONDS*1000);
                  	q.stop();
                  	flag = 0;
                  }
              } 	  	  
    	  	
    	  	if (q.whereTimerFuncName.equals(TimerFunctions.INTIMEINTERVAL))
    	  	  {
    	  	  	hour = Integer.parseInt(q.whereTimerFuncParams[0]);
              	minute = Integer.parseInt(q.whereTimerFuncParams[1]);
              	second = Integer.parseInt(q.whereTimerFuncParams[2]);
              	
              	hour2 = Integer.parseInt(q.whereTimerFuncParams[3]);
              	minute2 = Integer.parseInt(q.whereTimerFuncParams[4]);
              	second2 = Integer.parseInt(q.whereTimerFuncParams[5]); 
              	
              	if (flag == 0)
    	  	     {
    	  	        long sleep_interval = TimerFunctions.computeTimeInterval(hour,minute,second);  
                    Thread.currentThread().sleep(sleep_interval);
                    if (q.start_or_stop) q.start(); 
                    flag = 1;
    	  	     }
    	  	    else 
    	  	     {
    	  	       	long sleep_interval = TimerFunctions.computeTimeInterval(hour2,minute2,second2);  
                    Thread.currentThread().sleep(sleep_interval);
                    q.stop();
                    flag = 0;
    	  	     }
    	  	   }
    	  	     	  	
    	  	
    	  	if (q.whereTimerFuncName.equals(TimerFunctions.INDAYINTERVAL))
    	  	  {
    	  	    year = Integer.parseInt(q.whereTimerFuncParams[0]);
                month = Integer.parseInt(q.whereTimerFuncParams[1]);
              	day = Integer.parseInt(q.whereTimerFuncParams[2]);
              	
              	year2 = Integer.parseInt(q.whereTimerFuncParams[3]);
              	month2 = Integer.parseInt(q.whereTimerFuncParams[4]);
              	day2 = Integer.parseInt(q.whereTimerFuncParams[5]); 
              	
              	if (flag == 0)
    	  	      {
    	  	        long sleep_interval = TimerFunctions.computeDayTimeInterval(hour,minute,second);  
                    Thread.currentThread().sleep(sleep_interval);
                    if (q.start_or_stop) q.start(); 
                    flag = 1;
    	  	      }
    	  	    else 
    	  	      {
    	  	       	long sleep_interval = TimerFunctions.computeDayTimeInterval(hour2,minute2,second2);  
                    Thread.currentThread().sleep(sleep_interval);
                    q.stop();
                    flag = 0;
    	  	      }
    	  	  }
    	  	  
    	  	
    	  	if (q.whereTimerFuncName.equals(TimerFunctions.ONDAYOFWEEK))
    	  	  {
    	  	     date = q.whereTimerFuncParams[0];
    	  	         	  	     
    	  	     if (flag == 0)
    	  	      {
    	  	        long sleep_interval = TimerFunctions.computeDayOfWeekTimeInterval(date);  
                    Thread.currentThread().sleep(sleep_interval);
                    if (q.start_or_stop) q.start(); 
                    flag = 1;
    	  	      }
    	  	    else 
    	  	      {
    	  	       	Thread.currentThread().sleep(7*TimerFunctions.ONE_DAY_SECONDS*1000);
                    q.stop();
                    flag = 0;
    	  	      }
    	  	  }
    	  	  
    	  	if (q.whereTimerFuncName.equals(TimerFunctions.INDAYOFWEEKINTERVAL))
    	  	  {
    	  	    date = q.whereTimerFuncParams[0];
    	  	    date2 = q.whereTimerFuncParams[1];
    	  	    
    	  	    if (flag == 0)
    	  	      {
    	  	        long sleep_interval = TimerFunctions.computeDayOfWeekTimeInterval(date);  
                    Thread.currentThread().sleep(sleep_interval);
                    if (q.start_or_stop) q.start(); 
                    flag = 1;
    	  	      }
    	  	    else 
    	  	      {
    	  	       	long sleep_interval = TimerFunctions.computeDayOfWeekTimeInterval(date2);  
                    Thread.currentThread().sleep(sleep_interval);
                    q.stop();
                    flag = 0;
    	  	      }
    	  	  }
    	  	  
    	  	if (q.whereTimerFuncName.equals(TimerFunctions.EVERY))
    	  	  {
    	  	  	int time_interval = Integer.parseInt(q.whereTimerFuncParams[0]);
    	  	  	String time_unit = q.whereTimerFuncParams[1];
    	  	  	long sleep_interval = 0;
    	  	  	if (time_unit.equals(TimerFunctions.YEAR))
    	  	  	  sleep_interval = time_interval* (365*TimerFunctions.ONE_DAY_SECONDS*1000);
    	  	  	else if (time_unit.equals(TimerFunctions.MONTH))
    	  	  	       sleep_interval = (long)(time_interval* (30.5*TimerFunctions.ONE_DAY_SECONDS*1000));
    	  	  	     else if (time_unit.equals(TimerFunctions.DAY))
    	  	  	            sleep_interval = time_interval * TimerFunctions.ONE_DAY_SECONDS*1000;
    	  	  	          else if (time_unit.equals(TimerFunctions.HOUR))
    	  	  	                 sleep_interval = time_interval * 60 * 60 * 1000;
    	  	  	               else if (time_unit.equals(TimerFunctions.MINUTE))
    	  	  	                      sleep_interval = time_interval * 60 * 1000;
    	  	  	                    else if (time_unit.equals(TimerFunctions.SECOND))
    	  	  	                           sleep_interval = time_interval * 1000;
    	  	  	    	  	  	                         
    	  	  	Thread.currentThread().sleep(sleep_interval);
    	  	  	
    	  	  	if (!q.queryDropped) 
    	  	  	  { 
    	  	  	    q.eventStartTime = System.currentTimeMillis();
    	  	  	    q.start();
    	  	  	  }
    	  	  	
    	  	  }
    	      	  	  
    	  } 
         }  
    	 catch (Exception e) {e.printStackTrace();}
    	  
    } 
    
    public void initialize ()
      {
      	int year = 0,month = 0,day = 0;
    	int hour = 0,minute = 0,second = 0;
    	int year2 =0, month2 = 0, day2 = 0, day_of_week2 = 0;
    	int hour2 = 0, minute2 = 0, second2 = 0;
    	String date,date2;
    	
    	if (q.whereTimerFuncName.equals(TimerFunctions.ATTIME))
    	  {
    	  	hour = Integer.parseInt(q.whereTimerFuncParams[0]);
           	minute = Integer.parseInt(q.whereTimerFuncParams[1]);
            second = Integer.parseInt(q.whereTimerFuncParams[2]);
            if (TimerFunctions.atTime(hour,minute,second))
              	  if (q.start_or_stop)
              	    q.start();
          }
          
        if (q.whereTimerFuncName.equals(TimerFunctions.ONDAY))
          {
            year = Integer.parseInt(q.whereTimerFuncParams[0]);
           	month = Integer.parseInt(q.whereTimerFuncParams[1]);
           	day = Integer.parseInt(q.whereTimerFuncParams[2]);
            	
           	if (TimerFunctions.onDay(year,month,day))
           	  if (q.start_or_stop)
           	    {q.start(); flag = 1;}
          }
        
        if (q.whereTimerFuncName.equals(TimerFunctions.INTIMEINTERVAL))
    	  {
    	  	hour = Integer.parseInt(q.whereTimerFuncParams[0]);
            minute = Integer.parseInt(q.whereTimerFuncParams[1]);
           	second = Integer.parseInt(q.whereTimerFuncParams[2]);
              	
           	hour2 = Integer.parseInt(q.whereTimerFuncParams[3]);
            minute2 = Integer.parseInt(q.whereTimerFuncParams[4]);
           	second2 = Integer.parseInt(q.whereTimerFuncParams[5]); 
              	
           	if (TimerFunctions.inTimeInterval(hour,minute,second,hour2,minute2,second2) )
    	  	  if (q.start_or_stop)
    	  	    {q.start(); flag = 1;}
    	  }
        
        if (q.whereTimerFuncName.equals(TimerFunctions.INDAYINTERVAL))
    	  {
    	  	 year = Integer.parseInt(q.whereTimerFuncParams[0]);
             month = Integer.parseInt(q.whereTimerFuncParams[1]);
             day = Integer.parseInt(q.whereTimerFuncParams[2]);
              	
             year2 = Integer.parseInt(q.whereTimerFuncParams[3]);
             month2 = Integer.parseInt(q.whereTimerFuncParams[4]);
             day2 = Integer.parseInt(q.whereTimerFuncParams[5]); 
              	
             if (TimerFunctions.inDayInterval(year,month,day,year2,month2,day2))
    	  	   if (q.start_or_stop)
    	  	     {q.start(); flag = 1;}
    	  }
    	  
    	if (q.whereTimerFuncName.equals(TimerFunctions.ONDAYOFWEEK))
    	  {
    	  	 date = q.whereTimerFuncParams[0];
    	  	         	  	     
    	  	 if (TimerFunctions.onDayOfWeek(date))
    	  	    if (q.start_or_stop)
    	  	      {q.start(); flag = 1;}
    	  }
    	
    	if (q.whereTimerFuncName.equals(TimerFunctions.INDAYOFWEEKINTERVAL))
    	  {
    	  	 date = q.whereTimerFuncParams[0];
    	  	 date2 = q.whereTimerFuncParams[1];
    	  	    
    	  	 if (TimerFunctions.inDayOfWeekInterval(date,date2))
    	  	   if (q.start_or_stop)
    	  	     {q.start(); flag = 1;}
    	  	     
    	  }
    	  
    	if (q.whereTimerFuncName.equals(TimerFunctions.EVERY))
    	  if (q.start_or_stop)
    	    {
    	      q.eventStartTime = System.currentTimeMillis();    	      
    	      q.start();
    	    }
    	
      }    
   
}