/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.net.*;

/** 
 * Remote Cost Estimator (RCE) for the polling time on AXIS 2130 network cameras.   
 *  
 * @author Wenwei Xue (wwxue@cs.ust.hk)
 */
public class RCEPollingCamera {
	
  static final String statusURL = "/axis-cgi/com/ptz.cgi?query=position";
    
  static final String cameraIP = "192.168.1.201";
  
  static final String userPassEncoding = "d2F5bmU6bHVvbHVv";
  
  static final int num = 1; // = 100;
  
  public static void main(String argv [])
    {
      long pollingTime = 0;
      
      try
        {
          for (int i=0;i<num;i++)
            {
               long response_time = System.currentTimeMillis();
          	
               URL url = new URL("http://" + cameraIP + statusURL);
               URLConnection uc = url.openConnection();
               uc.setRequestProperty  ("Authorization", "Basic " + userPassEncoding);
               //uc.connect();
          
               BufferedReader br = new BufferedReader (new InputStreamReader(uc.getInputStream()));
               String line = br.readLine();
               while (line != null)
                  {
                  	//System.out.println(line); 
                    line = br.readLine();
                  }    	
                response_time = System.currentTimeMillis() - response_time;
                //System.out.println(response_time);
                pollingTime += response_time;
                br.close();
             }
      
          System.out.println("Polling Time: " + (pollingTime / num) + " ms.");                 
        }
      catch (Exception e) {e.printStackTrace();}          		
    }	
 	
}