/*

* Copyright (c) 2006, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

import java.io.*;
import java.net.*;
import java.util.*;

/** 
 * Remote Cost Estimator (RCE) for the "receiveTextMessage" atomic operation on cell phones.
 *
 * @author Wenwei Xue (wwxue@cs.ust.hk)
 */
public class RCESMS	
{
    String serverName = "www.csoft.co.uk";  // "www2.csoft.co.uk"
    int serverPort = 80;

    String  requestURI = "/sendsms";
    String query = "UN=xxxxx.xxxxx&PIN=xxxxxxxx";

    String requestMethod = "POST";  // "GET"
    
    String protocol = "HTTP/1.1";
    
    // for POST method
    String contentTypeHeader = "Content-Type: application/x-www-form-urlencoded; charset=\"utf-8\"";
    
    String SUC_RESPONSE_CODE = "01";
    
    RCESMS (String message, String phoneno)
      {
      	query += "&M=" + message;
      	query += "&N=" + "852" + phoneno;      	
      }
    
    public boolean process() 
      {
        Socket socket = null;
	    
	    int reqId = 0;
	    char space = ' ';
	
    	String newLine = "\r\n";
    	
    	boolean suc = false;
		
 	    try 
 	      {
 	    	String request = assembleHTTPRequest();
 	    	
 	    	socket = new Socket(serverName,serverPort);
 	    	
 	    	InputStream ins = sendRequest(request,socket);	
 	    	
 	    	suc = receiveResponse(ins);	  
          }  
	     catch (Exception e) {e.printStackTrace();}
	     
	     return suc;	
      } 
    
    public String assembleHTTPRequest() 
      {
    	char space = ' ';
        String newLine = "\r\n";
        String slash = "/";

    	StringBuffer sb = new StringBuffer(requestMethod);
        
        sb.append(space);
        
        sb.append(requestURI);

        if (query != null && requestMethod.equalsIgnoreCase("GET")) 
          {
            sb.append('?');
            sb.append(query.trim());
          }

	    sb.append(space);
	    sb.append(protocol);
        sb.append(newLine);

	    if (query != null && requestMethod.equalsIgnoreCase("POST")) 
	      {
	      	sb.append(contentTypeHeader);
	        sb.append(newLine);
	    
	        sb.append("Content-Length: ");
	        sb.append(query.length());	    
	        sb.append(newLine);

	        sb.append(newLine);
	        sb.append(query);	    
          }
          
        sb.append(newLine);
	
        return sb.toString();
      } 
    
    
    public InputStream sendRequest (String request,Socket socket) 
      {
    	InputStream responseIns = null;
        PrintWriter sender = null;

        try 
          {
            sender = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())));
            responseIns = socket.getInputStream();
            sender.print(request);
            sender.flush();
          }
        catch (Exception e) {e.printStackTrace();}
        
        return responseIns;
      } 
    
    
    public boolean receiveResponse(InputStream ins)  
      {
    	boolean suc = false;
    	
        try 
          {
            BufferedReader br = new BufferedReader(new InputStreamReader(ins));

            String line;            
                      
            while (!(line=br.readLine()).equals("")) 
	          {
	          	// System.out.println(line);
	          }	          
	          
	        line = br.readLine().trim();
	        // System.out.println(line);	       
	        
	        if (line.equals(SUC_RESPONSE_CODE))
	          suc = true;
	        else 
	          suc = false;	        
	        
	        br.close();		           
	      }
        catch (Exception e) {e.printStackTrace();}
        
        return suc;       
    } 
    
    public static void main (String argv [])
      {
      	long startTime, endTime;
      	long cost = 0;
      	int num = 1; // 100;
      	
      	try
      	  {
      	  	for (int i=0;i<num;i++) 
      	  	  {
      	  	    startTime = System.currentTimeMillis();
      	  	
      	    	String message = "";
      	  	
      	  	    BufferedReader br = new BufferedReader (new FileReader("alarm.txt"));
      	  	    String line;
      	  	
      	  	    while ( (line = br.readLine())!= null)
      	  	      message += line + "\r\n";
      	  	    //System.out.println(message);
      	  	
      	  	    RCESMS rce_sms = new RCESMS(message,"85297585242");
      	  	
      	  	    rce_sms.process();
      	  	
      	  	    endTime = System.currentTimeMillis();
      	  	
      	  	    // System.out.println("Estimated cost: " + (endTime - startTime));
      	  	    cost += endTime - startTime;
      	  	  }
      	  	System.out.println("Estimated cost: " + (cost/num) + " ms.");
      	  }
      	catch (Exception e) {e.printStackTrace();}
      }    
}
